#! /usr/bin/python
# -------------------------------------------------------------------
# Copyright (C) 2015 by Intevation GmbH
# Author(s):
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GNU GPL (>=v2)
# Read the file COPYING coming with the software for details.
# -------------------------------------------------------------------

"""Script to coordinate the import of data into the evaluation databases."""

import sys
import os
from contextlib import contextmanager
import subprocess
import shutil
import tempfile
import datetime
import json
import logging
import sqlite3
from optparse import OptionParser


here = os.path.dirname(__file__)

log = logging.getLogger("import-jmd")

EVALUATION_TRANSFER_DIR = os.path.join(here, os.pardir, os.pardir,
                                       "evaluation-transfer")
IMPORTSQLITE = os.path.join(EVALUATION_TRANSFER_DIR, "import-sqlite.py")
EXPORTSQLITE = os.path.join(EVALUATION_TRANSFER_DIR, "export-sqlite.py")


class Relation(object):

    """Information about a relation that is to be imported/exported"""

    def __init__(self, name, parent=None):
        self.name = name
        self.parent = parent

    @property
    def table_name(self):
        return self.name + "_tbl"

    @property
    def with_parent(self):
        result = self.name
        if self.parent is not None:
            result += ":" + self.parent
        return result


class Config(object):

    """Configuration settings.

    Some of the settings are different for plain jmd and jmd2start.
    These settings are represented dictionaries and are described as
    "mapping from project name to ..." in the description of the
    attributes below. Here, "project" stands for the distinction between
    "jmd" and "jmd2start".

    Attributes:

    dbs: mapping from project name to a list of traeger databases, each
         specified as a triple of the form [NAME, PORT, DBNAME]. If the
         mapping contains a list for "jmd2start" the names in that list
         should also be found in the "jmd" list.

    bund_dbs: Like dbs, but for the bund-level databases.

    bund_templates: mapping from project name to the file name of the
                    corresponding template sqlite database. The
                    databases should list the correct traeger databases
                    from which to export (jmd2start vs. jmd).

    relations: list of the the relations to export/import in addition to
               the standard relations. Each item may either be a
               relation name or a pair of a relation name and the name
               of it's parent relation (e.g. for nested repeat groups or
               multi-select fields). The _tbl suffix must not be
               included in the relation names.

               This list of relations is used to construct the value of
               the --relations parameter for both import-sqlite.py and
               export-sqlite.py.

    blacklist: List of database fields not to copy. The value of the
               --blacklist parameter of export-sqlite.py is derived from this

    owner: The user id to use as the owner of the imported data.

    datadir: directory under which the input sqlite files can be found.

    logdir: directory for the log files.
    """

    def __init__(self, dbs, bund_dbs, bund_templates, relations, blacklist,
                 owner, datadir, logdir):
        self.dbs = dbs
        self.bund_dbs = bund_dbs
        self.bund_templates = bund_templates
        self.relations = relations
        self.blacklist = blacklist
        self.owner = owner
        self.datadir = datadir
        self.logdir = logdir


def read_config(config_file):
    """Read the configuration from config_file.
    The file should contain a JSON serialization of a dictionary usable
    as keyword arguments to Config.
    """
    global config
    with open(config_file) as infile:
        decoded = json.load(infile)
        decoded["relations"] = convert_relations(decoded["relations"])
        config = Config(**decoded)


def convert_relations(config_relations):
    relations = []
    for description in config_relations:
        if isinstance(description, basestring):
            args = (description,)
        else:
            args = description
        relations.append(Relation(*args))
    return relations


def dbname(fkz):
    """Return the name of the postgres database for a given FKZ.
    This follows the mpuls convention of prefixing the FKZ with 'db_'
    and appending '_db'.
    """
    return "ka_%s_db" % (fkz,)


def logfilename(fkz, prefix="import"):
    """Return the logfile to use for FKZ specific messages.
    This function assumes that the configuration is available as the
    global variable config.
    """
    return os.path.join(config.logdir, "%s%s.log" % (prefix, fkz,))


def compressed_database(fkz, date):
    """Return the absolute filename of the database for FKZ and date.
    This function assumes that the configuration is available as the
    global variable config. The returned filename will only be absolute
    if the configuration setting 'datadir' as a string with an absolute
    filename.
    """
    return os.path.join(config.datadir, str(fkz), "jmd-%s.db.bz2" % (date,))


def sqlite_filename(workdir, fkz):
    return os.path.join(workdir, "jmd-%s.db" % (fkz,))


def bund_sqlite_filename(workdir, name):
    return os.path.join(workdir, "bund-%s.db" % (name,))


@contextmanager
def sqlite_transaction(sqlite_file):
    con = sqlite3.connect(sqlite_file)
    try:
        yield con.cursor()
        con.commit()
    finally:
        con.close()


def unzip_database(workdir, fkz, date, owner):
    """Unzip an sqlite database into workdir.

    The name of the compressed input file is determined by
    compressed_database based on the parameters fkz and date, the name
    of the output file by sqlite_filename based on the parameters
    workdir and fkz.

    After uncompression, the bearbeiter_id of all cases in the
    uncompressed database is set to owner.
    """
    compressed = compressed_database(fkz, date)
    target = sqlite_filename(workdir, fkz)
    log.debug("start decompressing %r to %r", compressed, target)

    subprocess.check_call(["bunzip2", "-c", compressed],
                          stdout=open(target, "wb"))

    # Setting the owner must be done in the SQLite databases because the
    # bearbeiter_id field may not be NULL in the databases and in some
    # databases (the st-databases) there's a foreign-key constraint on
    # bearbeiter_id.
    with sqlite_transaction(target) as cur:
        cur.execute("UPDATE master_tbl SET bearbeiter_id = ?", (owner,))

    log.debug("finished decompressing %r to %r", compressed, target)

    return target

def unzip_databases(workdir, date, dbs):
    """Unzip the sqlite input databases for the given DBs.

    The parameter dbs should be an iterable of triples of the form
    (NAME, PORT, DBNAME). This list can be taken more or less directly
    from the configuration. The NAME part of the triple is used together
    with the parameters workdir and date to find the input files.

    The return value is a dictionary mapping the NAMEs from the db
    triples to the name of uncompressed sqlite file.
    """
    log.debug("start decompressing sqlite files")
    unzipped = {}
    for db in dbs:
        name = db[0]
        unzipped[name] = unzip_database(workdir, name, date, config.owner)
    log.debug("finished decompressing sqlite files")
    return unzipped


def create_indices(sqlite_file):
    # Create indices on master_id for all the relevant tables. This
    # improves the delete_from_master substantially.
    log.debug("start creating indices in %r", sqlite_file)
    tables = (["ka_status_tbl", "nm_tags_cases", "ka_logbuch_tbl"]
              + [relation.table_name for relation in config.relations])
    with sqlite_transaction(sqlite_file) as cur:
        for table in tables:
            cur.execute("CREATE INDEX IF NOT EXISTS %s ON %s(master_id);"
                        % (table + "_master_id_idx", table))
    log.debug("finished creating indices in %r", sqlite_file)


def filter_master_tbl(sqlite_file, cond):
    """Delete all tuples from master_tbl not satisfying the condition cond.
    """
    log.debug("start deleting from %r where not (%r)", sqlite_file, cond)
    with sqlite_transaction(sqlite_file) as cur:
        # switch on foreign_keys so that deleting rows from master_tbl
        # automatically deletes the rows in other tables that refer to
        # them
        cur.execute("PRAGMA foreign_keys = true;")
        cur.execute("DELETE FROM master_tbl WHERE NOT (%s);" % cond)
    log.debug("finished deleting from %r where not (%r)", sqlite_file, cond)


def split_jmd2start(combined_db):
    """Extract the jmd2start data into a separate database."""
    log.debug("splitting jmd2start from %r", combined_db)
    combined_base = os.path.splitext(combined_db)[0]
    jmd2start = combined_base + "-jmd2start.db"
    create_indices(combined_db)
    shutil.copyfile(combined_db, jmd2start)

    # zielgruppe may have the following values:
    #   -1 = not specified yet
    #    0 = JMD
    #    1 = jmd2start
    # The jmd2start DB should contain only the jmd2start cases, i.e. the
    # ones where zielgruppe == 1. The combined_db all the others.
    filter_master_tbl(jmd2start, "zielgruppe == 1")

    log.debug("finished splitting jmd2start from %r", combined_db)
    return (combined_db, jmd2start)


def start_import_db(dbname, port, sqlitefile, logfile):
    relations = ",".join(r.table_name for r in config.relations)
    command = [IMPORTSQLITE, "--truncate", "-p", repr(port),
               "-r", relations, "-o", dbname,
               "-i", sqlitefile]
    log.debug("Starting subprocess %r", command)
    return subprocess.Popen(command, stdout=logfile, stderr=subprocess.STDOUT)


def start_imports(dbs):
    """Start processes to import the sqlite databases into PostgreSQL.
    The parameter should be a list of (PORT, FKZ, SQLITE_FILENAME)
    triples, where PORT is the port of the PostgreSQL database, FKZ is
    the mpuls specific part of the PostgreSQL database name (see the
    dbname function) and SQLITE_FILENAME ist the filename of the sqlite
    database.
    """
    subprocs = []
    for port, fkz, sqlitefile in dbs:
        log.debug("Starting import for %s", fkz)
        logfile = logfilename(fkz)
        proc = start_import_db(dbname(fkz), port, sqlitefile,
                               open(logfile, "ab"))
        subprocs.append((fkz, proc, logfile))

    return subprocs


def import_all(unzipped_dbs):
    """Import all sqlite databases described in unzipped_dbs.
    The parameter is passed through to start_imports, so see that for
    how the information is represented.
    """
    subprocs = start_imports(unzipped_dbs)

    failed = False
    for fkz, proc, logfile in subprocs:
        log.debug("Waiting for %s", fkz)
        ret = proc.wait()
        if ret != 0:
            failed = True
            log.error("Could not import into %r (exit code %d, logfile %r)",
                      fkz, proc.returncode, logfile)
        else:
            log.debug("%s imported", fkz)

    if failed:
        exit(1)


def import_traeger(unzipped_dbs):
    """Import the traeger databases described in unzipped_dbs.
    This function is just a wrapper around import_all that adds traeger
    specific logging output.
    """
    log.debug("start importing traeger data")
    import_all(unzipped_dbs)
    log.debug("finished importing traeger data")


def split_jmd2start_traeger(unzipped_dbs):
    """Split sqlite databases into jmd and jmd2start parts.

    The unzipped_dbs parameter should be a mapping from the db names to
    sqlite file names as returned by unzip_databases. For each of the
    sqlite databases referenced, this function creates a copy with only
    the jmd2start cases and then deletes the jmd2start cases from the
    original sqlite database.

    The return value is a pair of two dictionaries of the same format as
    the input dictionary. The first dictionary of the pair references
    the JMD sqlite files, the second the ones for jmd2start.
    """
    jmd = {}
    jmd2start = {}
    for name, sqlite_file in unzipped_dbs.items():
        jmd[name], jmd2start[name] = split_jmd2start(sqlite_file)
    return (jmd, jmd2start)


def build_traeger_import_info_for(project, sqlite_files):
    dbs = []
    for name, port, fkz in config.dbs.get(project, ()):
        sqlite_file = sqlite_files.get(name)
        if sqlite_file is not None:
            dbs.append((port, fkz, sqlite_file))
    return dbs


def build_traeger_import_info(split_traeger):
    """Convert split_jmd2start_traeger result into input for import_traeger.
    """
    jmd_dbs, jmd2start_dbs = split_traeger
    return (build_traeger_import_info_for("jmd", jmd_dbs)
            + build_traeger_import_info_for("jmd2start", jmd2start_dbs))


def export_traeger(dbs, target_sqlite, logfile):
    # Each database cluster must be accessed at most once in order to
    # make sure that each database listed in the template DB is exported
    # only once. So we use a set of ports to make sure each port is
    # accessed only once.
    ports = ",".join(set(str(port) for name, port, dbname in dbs))

    relations = ",".join(r.with_parent for r in config.relations)
    command = [EXPORTSQLITE, "-o", target_sqlite, "-p", ports,
               "-r", relations, "--keep-fkz",
               "--blacklist", ",".join(config.blacklist)]
    log.debug("Running sub-process: %r", command)
    subprocess.check_call(command, stdout=logfile, stderr=subprocess.STDOUT)


def export_bund(dbs, templates, workdir):
    log.debug("start exporting bund data")

    exported = {}

    for name in dbs:
        log.debug("start exporting bund data for %r dbs", name)
        transfer_sqlite = bund_sqlite_filename(workdir, name)
        shutil.copyfile(templates[name], transfer_sqlite)
        export_traeger(dbs[name], transfer_sqlite,
                       open(logfilename("bund", prefix="export"), "ab"))
        log.debug("finished exporting bund data for %r dbs", name)
        exported[name] = transfer_sqlite

    return exported

    log.debug("finished exporting bund data")


def re_use_bund(dbfile, workdir):
    transfer_sqlite = bund_sqlite_filename(workdir)
    log.debug("start copying %r to %r", dbfile, transfer_sqlite)
    shutil.copyfile(dbfile, transfer_sqlite)
    log.debug("finished copying %r to %r", dbfile, transfer_sqlite)
    return transfer_sqlite


def build_bund_import_info(sqlite_files):
    bund_dbs = []
    for project, dbs in config.bund_dbs.items():
        sqlite_file = sqlite_files.get(project)
        if sqlite_file is not None:
            bund_dbs.extend((port, fkz, sqlite_file) for port, fkz in dbs)
    return bund_dbs


def import_bund(sqlite_files):
    log.debug("start importing bund data")
    import_all(build_bund_import_info(sqlite_files))
    log.debug("finished importing bund data")


def parse_options():
    parser = OptionParser()
    parser.set_defaults(log_level=logging.INFO,
                        date=datetime.date.today().isoformat(),
                        keep_temp_dir=False)
    parser.add_option("--log-level", help="Logging level threshold.")
    parser.add_option("--date", help="Date used in the database names.")
    parser.add_option("--config-file", help="Configuration file (MANDATORY!)")
    parser.add_option("--keep-temp-dir", action="store_true",
                      help=("Do not delete the temporary directory used for"
                            " temporary files"))

    opts, args = parser.parse_args()

    if opts.config_file is None:
        print >>sys.stderr, "Please specify a configuration file"
        parser.print_usage(sys.stderr)
        exit(1)

    return (opts, args)


def main():
    opts, args = parse_options()

    read_config(opts.config_file)

    if not os.path.exists(config.logdir):
        os.makedirs(config.logdir)

    logging.basicConfig(stream=sys.stderr, level=opts.log_level,
                        format="%(asctime)s:%(levelname)s:%(name)s:%(message)s")

    workdir = tempfile.mkdtemp(prefix="jmdimport")
    try:
        log.info("Temporary directory: %r", workdir)
        unzipped_traeger = unzip_databases(workdir, opts.date,
                                           config.dbs["jmd"])
        if "jmd2start" in config.dbs:
            split_traeger = split_jmd2start_traeger(unzipped_traeger)
        else:
            split_traeger = (unzipped_traeger, {})
        import_traeger(build_traeger_import_info(split_traeger))
        transfer_sqlite = export_bund(config.dbs, config.bund_templates,
                                      workdir)
        import_bund(transfer_sqlite)
    finally:
        if not opts.keep_temp_dir:
            shutil.rmtree(workdir)


main()
