# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008, 2010 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls JMD has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
#

import logging
import datetime

from pylons import app_globals as g
from formed.meta import UNKNOWN_DATE
from formed.meta.data import EnumeratedNode
from formed.instance.backends.xmlimport import XMLImportException

import mpulsweb.lib.helper.filters as F
import mpulsweb.lib.helpers as h
from mpulsweb.lib.db import db
from mpulsweb.lib.translation import _
from mpulsweb.model.agencysettings import Agency
from mpulsweb.model.case import MpulsCase, MpulsCaseFactory, \
     MpulsCaseOverview, MpulsCaseBundle, MpulsCaseState, MPulsXMLImporter
from mpulsweb.model.state import PhaseStateMachine

from jmdweb.model.statement import DiscretionStatement


log = logging.getLogger(__name__)



class CaseBundle(MpulsCaseBundle):

    pass


class CaseOverview(MpulsCaseOverview):

    pass


class State(MpulsCaseState):

    pass


class CaseFactory(MpulsCaseFactory):

    def __init__(self, case_cls=None):
        if case_cls is None:
            # FIXME Case is not defined or imported here. Should it be a
            # MpulsCase???
            case_cls = Case
        self.case_cls = case_cls

    def get_state_machine(self):
        return JMDPhaseStateMachine()


class Case(MpulsCase):

    def getDiscretionStatement(self):
        self.discretion_statement = None
        """Returns the discretion statement of the case"""
        if not self.discretion_statement is None:
            return self.discretion_statement
        try:
            # Create privacy_statement
            self.discretion_statement = DiscretionStatement(self, Agency())
            return self.discretion_statement
        except:
            self.discretion_statement = None
            raise
        return None

    @classmethod
    def new_overview_field(cls, name, label=None, style=None, getter=None,
                           formatter=None, max_length=None, roles=None):
        if name == "geburtsdatum" and label == "Alter":
            def formatter(birthday):
                if birthday is None or birthday == UNKNOWN_DATE:
                    return "-/-"
                return h.get_age(datetime.date.today() - birthday)
            style = "table_width_mid_s"
        elif name.endswith("_agg"):
            # backwards compatibility with older configurations which
            # had fieldnames like mobilfunknummer_agg and adresse_plz_agg
            name = name[:-4]

        return MpulsCase.new_overview_field(
            name, label, style,
            getter=getter,
            formatter=formatter, max_length=max_length, roles=roles)

    def abort_case(self):
        """Abort running CM or 'Beratung'."""
        with db.transaction() as cur:
            cur.execute("SELECT abort_case(%s);", (self.id,))



class JMDPhaseStateMachine(PhaseStateMachine):

     def _load_case(self, case):
         """Load case and update the phase list for non-jmd2start agencies.
         This method extends the base class method to remove the
         jmd2start specific fields from the phaselist for non-jmd2start
         agencies after calling the base class method. This is done in
         order not to show those fields in the phase overview where they
         would be confusing for users of non-jmd2start agencies.
         """

         super(JMDPhaseStateMachine, self)._load_case(case)
         if not Agency().get_value("jmd2start"):
             remove_jmd2start_fields(self.phaselist)


def remove_jmd2start_fields(phaselist):
    """Removes the jmd2start specific fields from phaselist.
    phaselist is modified in-place.
    """
    phase = phaselist.get_phase(1)
    for i in range(len(phase.fields) - 1, -1, -1):
        if (phase.fields[i].name.partition(":")[0] in ("zielgruppe",
                                                       "aufenthaltspapier",
                                                       "aufenthaltspapier_eintritt")):
            del phase.fields[i]



class JMDXMLImporter(MPulsXMLImporter):

    def import_case(self, factory, cursor, components):
        self.update_art_cm_ende(components)
        tree = super(JMDXMLImporter, self).import_case(factory, cursor,
                                                       components)

        # In JMD, datum_ea_start must be given. There's a rule for that,
        # but that rule is only evaluated if the value would change due
        # to the import. In particular, it won't be evaluated when
        # importing a new case that does not set datum_ea_start.
        #
        # We do the check here, so that the database transaction has not
        # been committed yet. The XMLImportException raised if the value
        # is None will be caught by the controller and the associated
        # message is shown to the user.
        datum_ea_start = tree.getAllItemsForName("datum_ea_start")[0].getValue()
        if datum_ea_start is None:
            raise XMLImportException("datum_ea_start muss gesetzt sein")

        return tree

    def update_art_cm_ende(self, components):
        """Change obsolete value 2 of art_cm_ende to 0.
        This corresponds to the DB upgrade script
        application-db/upgrade/1.9.4.4/10art_cm_ende.sql
        """
        instance = components["master"]
        if instance.get("art_cm_ende") == 2:
            log.info("XMLImporter: correcting art_cm_ende from 2 to 0")
            instance.set_value("art_cm_ende", 0)

    def version_supported(self, version):
        """Overwrite inherited method to support a range of versions."""
        current_version = self.structure_tree.get_version()
        accepted = set([current_version])
        if current_version == ("http://mpuls.intevation.de/formed/jmd-case/"
                               "2020-03-06/17:22"):
            accepted.add("http://mpuls.intevation.de/formed/jmd-case/"
                         "2017-06-06/12:00")
        if version not in accepted:
            raise XMLImportException(_("The XML file has unsupported version"
                                       " '%s'. Supported are: %s")
                                     % (version, ", ".join(accepted)))

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
