# -*- coding: utf-8 -*-
import logging

import formencode

from cgi import escape

from sets import Set

from pylons import app_globals as g, response
from pylons.controllers.util import redirect

from formed.meta.data import PageNode

from mpulsweb.lib.base import c, render, request, session, DownloadController, \
    render_htmlfill, restrict_to_POST, abort_unless_POST
from mpulsweb.lib.export import FormletterExport
from mpulsweb.lib.helpers import url, format_object
from mpulsweb.lib.helper.filters import NA
from mpulsweb.lib.security import checkRole, check_feature
from mpulsweb.lib.translation import _, ungettext
from mpulsweb.lib.validators import BundleActionForm, SetBundleEditorForm, \
    SetBundleStandinForm, ExportSelectorValidator
from mpulsweb.model.case import MpulsCaseBundle
from mpulsweb.model.phase import ConsistenceCheckException
from mpulsweb.model.user import UserGroupList, UserListObject, UserObject
from mpulsweb.controllers.case_overview import CaseOverviewController


log = logging.getLogger(__name__)


class CaseBundleController(DownloadController):

    def index(self):
        return "Hello world."

    def overview(self):
        return CaseOverviewController().index()

    def _get_bundle_action_validator(self):
        return BundleActionForm()

    @restrict_to_POST
    def bundleAction(self):
        # Check submitted fields
        validator = self._get_bundle_action_validator()
        log.debug(request.params)
        form_result = validator.to_python(request.params)
        log.debug(form_result)

        # Check if the user has selected any cases
        if (len(form_result.get('case_id', [])) <= 0
                and not form_result.get('all_cases')):
            return self._empty_caselist()

        # Build case bundle and Delegate to concrete action
        return self._bundle_action(form_result.get('action'),
                                   self._build_casebundle(form_result))

    def _bundle_action(self, action, case_bundle):
        # No action
        if action == 'no_action':
            return self.no_action()
        if action == 'markdelete':
            # Fiter out case which are not open(1) or inactive (2)
            case_bundle = self._filterBundle(case_bundle,
                                             allowedstates=(1, 2))
            if case_bundle.isEmpty():
                return self._empty_caselist()
            session['casebundle'] = case_bundle
            session.save()
            return self.markDelete(0)
        if action == 'delete':
            # Fiter out case which are not marked for deletion
            case_bundle = self._filterBundle(case_bundle,
                                             allowedstates=(3, 5))
            if case_bundle.isEmpty():
                return self._empty_caselist()
            session['casebundle'] = case_bundle
            session.save()
            return self.delete(0)
        if action == 'markanonymize':
            # Fiter out case which are not open(1) or inactive (2)
            # or not anonymizable
            case_bundle = self._filterBundle(case_bundle,
                                             allowedstates=(1, 2),
                                             filter_notanonymizable=True)
            if case_bundle.isEmpty():
                return self._empty_caselist()
            session['casebundle'] = case_bundle
            session.save()
            return self.markAnonymize(0)
        if action == 'anonymize':
            # Fiter out case which are not open(1) or inactive (2)
            # or not anonymizable
            case_bundle = self._filterBundle(case_bundle,
                                             allowedstates=(4,),
                                             filter_notanonymizable=True)
            if case_bundle.isEmpty():
                return self._empty_caselist()
            session['casebundle'] = case_bundle
            session.save()
            return self.anonymize(0)
        if action == 'restore':
            # Filter out cases which aren't marked for anonymization (4),
            # or marked for deletion (3)
            case_bundle = self._filterBundle(case_bundle,
                                             allowedstates=(3, 4))
            if case_bundle.isEmpty():
                return self._empty_caselist()
            session['casebundle'] = case_bundle
            session.save()
            return self.restore(0)
        if action == 'standin':
            # filter out cases where the user is not the editor and
            # therefor is not allowed to set the standin.
            user = session['USER_AUTHORIZED']
            filtered = []
            for case in case_bundle.getCases():
                if (case.getEditor().id != user.id and not user.isAdmin()):
                    filtered.append(case.id)

            # Fiter out case which are not open(1) or inavtice (2)
            case_bundle = self._filterBundle(case_bundle, (1, 2), filtered)
            if case_bundle.isEmpty():
                return self._empty_caselist()
            session['casebundle'] = case_bundle
            session.save()

            group_list = UserGroupList()
            user_list = UserListObject()
            group_filter = [u.gid for u in user_list.getAdminList()]
            group_filter.append(user.gid)
            c.group_list = group_list.getGroupList(group_filter)
            c.case_list = case_bundle.getCases()
            c.current_user = user

            return render('/casebundle/standin.mako')
        if action == 'transfer':
            # Fiter out case which are not open(1) or inavtice (2)
            user = session['USER_AUTHORIZED']
            case_bundle = self._filterBundle(case_bundle, (1, 2))
            if case_bundle.isEmpty():
                return self._empty_caselist()
            session['casebundle'] = case_bundle
            session.save()

            user_list = UserListObject()
            user_filter = [u.id for u in user_list.getAdminList()]
            user_filter.append(user.id)
            c.user_list = user_list.getUserList(user_filter)
            c.case_list = case_bundle.getCases()

            return render('/casebundle/editor.mako')

        if action.startswith("export"):
            # allow export of cases that are active (1), marked for
            # anonymization (4) or anonymized (5).  For historical
            # reasons, there's also the old now unused state 2 in
            # the filter tuple.
            case_bundle = self._filterBundle(case_bundle, (1, 2, 4, 5))
            if case_bundle.isEmpty():
                return self._empty_caselist()
            session['casebundle'] = case_bundle
            session.save()
            export_format = action[6:]
            if export_format in ("XML", "CSV", "XLS", "EXLS"):
                return self.export(export_format)

        if action.startswith("formletter"):
            # allow formletter of cases that are active (1). For historical
            # reasons, there's also the old now unused state 2 in
            # the filter tuple.
            case_bundle = self._filterBundle(case_bundle, (1, 2))
            if case_bundle.isEmpty():
                return self._empty_caselist()
            session['casebundle'] = case_bundle
            session.save()
            letter_id = action.split(':')[1]
            return self.formletter(letter_id)

        if action == 'evaluate':
            case_bundle = self._filterBundle(case_bundle, (1, 4, 5))
            if case_bundle.isEmpty():
                return self._empty_caselist()

            session['casebundle'] = case_bundle
            session["evaluation_ids"] = case_bundle.listDatasetIds()
            session["id_field"] = session.get('id_field_candidate')
            session["sql_where"] = session.get('sql_where_candidate')
            session.save()
            num = len(case_bundle)
            c.success_text = \
                ungettext("%d case was marked for evaluation."
                          " All following evaluations will be limited"
                          " to that case.",
                          "%d cases were marked for evaluation."
                          " All following evalutations will be limited"
                          " to those cases.", num) % num

            return self.overview()

    def _filterBundle(self, case_bundle, allowedstates, filtered_cases=None,
                      filter_notanonymizable=False):
        """Returns a filtered CaseBundle. Filtering can be done on
        states and on case_ids.
        """
        filtered = []
        for case in case_bundle.getCases(filtered_cases):
            if filter_notanonymizable:
                try:
                    case.check_anonymizeability()
                except ConsistenceCheckException:
                    continue
            state = case.getState()
            if state.getState() in allowedstates:
                filtered.append(case.id)
        case_bundle = MpulsCaseBundle(filtered)
        return case_bundle

    def _funcFilterBundle(self, case_bundle, func):
        filtered = []
        for case in case_bundle.getCases(None):
            if func(case):
                filtered.append(case.id)
        return MpulsCaseBundle(filtered)

    def _buildCaseList(self, case_bundle):
        case_list = case_bundle.getCases()
        return build_case_list(case_list)

    def _build_casebundle(self, options):
        if options.get('all_cases') > 0:
            ids = options["all_ids"]
        else:
            ids = options.get("case_id")
        return MpulsCaseBundle(ids)

    def _empty_caselist(self):
        c.url_ok = url(controller='case_overview', action='index')
        c.dialog_title = _("No valid cases in the selection!")
        c.dialog_text = _(u"The selected action can not be executed"
                          u"\non any of the cases in your selection."
                          u" Please make sure that the cases are"
                          u"\nin a state from which you can execute the "
                          u"selected action.")
        return render('/casebundle/dialogs/error.mako')

    #
    # ACTIONS
    #
    def no_action(self):
        c.dialog_title = _("No action specified for the selection")
        c.dialog_text = _("""You have not selected any action for the
selection. Please click OK to go to the case overview and select there
an action from the drop down menu.""")
        c.url_ok = url(controller='case_overview', action='index')
        return render('/casebundle/dialogs/error.mako')

    def _get_bundle_or_redirect(self):
        """Get the case bundle from the session, redirect to overview if missing.
        """
        case_bundle = session.get('casebundle')
        if case_bundle is None:
            redirect(url(controller='case_overview', action='index'))
        return case_bundle

    # DELETE
    @checkRole(('cm'))
    def markDelete(self, confirmed):
        confirmed = self._checkBool(confirmed)
        c.context = "../main.mako"
        case_bundle = self._get_bundle_or_redirect()
        if confirmed == 1:
            abort_unless_POST()
            num = case_bundle.markDelete()
            c.success_text = (ungettext(u"%s case was successfully cleared for"
                                       u" deletion.",
                                       u"%s cases were successfully cleared for"
                                       u" deletion.", num) % num)
            del session['casebundle']
            return self.overview()
        else:
            c.context = "../main.mako"
            c.dialog_title = ungettext(u'Delete case?', u'Delete cases?',
                                       len(case_bundle.getCases()))
            c.dialog_text = (ungettext('Do you really want to delete the'
                                       ' following case?%s',
                                       'Do you really want to delete the'
                                       ' following cases?%s',
                                       len(case_bundle.getCases()))
                             % "\n".join(self._buildCaseList(case_bundle)))
            c.url_yes = url(controller='case_bundle', action='markDelete',
                            confirmed=1)
            last_id = case_bundle.case_list[-1]
            c.url_no = url(controller='case_overview',
                           action='index', anchor=last_id)
            return render('/casemanagement/dialogs/confirm.mako')

    @checkRole(('admin'))
    def delete(self, confirmed):
        confirmed = self._checkBool(confirmed)
        c.context = "../main.mako"
        case_bundle = self._get_bundle_or_redirect()
        total = case_bundle.numDatasets()

        if not confirmed:
            c.dialog_title = ungettext(u'Delete case?', u'Delete cases?',
                                       total)
            c.dialog_text = (ungettext('Do you really want to delete the'
                                       ' following case?%s',
                                       'Do you really want to delete the'
                                       ' following cases?%s',
                                       total)
                             % "\n".join(self._buildCaseList(case_bundle))
                             + _("Warning!"
                                 " Deleting cases will erase all data of the"
                                 " case irrecoverably!<br>\n"
                                 "Only delete the case if this case is"
                                 " <b>not needed for evaluations</b>"
                                 " anymore.<br><br>\n"
                                 "Do you want to delete the case from the"
                                 " server?""")
                             + "<br><br>")
            c.url_yes = url(controller='case_bundle', action='delete',
                            confirmed=1)
            last_id = case_bundle.case_list[-1]
            c.url_no = url(controller='case_overview',
                           action='index', anchor=last_id)
            return render('/casemanagement/dialogs/confirm.mako')

        abort_unless_POST()
        successes = case_bundle.delete()
        failures = total - successes

        if not failures:
            c.dialog_title = ungettext(u"Case successfully deleted.",
                                       u"Cases successfully deleted.",
                                       total)
            c.dialog_text = (
                ungettext(u"The selected case was successfully deleted.",
                          u"All selected cases were successfully deleted.",
                          total)
                + " " + _("Please click OK to continue."))
            template = '/casebundle/dialogs/success_delete_admin.mako'
        elif successes > 0:
            # successes != total and successes > 0 means total
            # is also greater than 1, so we can always assume the
            # plural when talking about all selected cases
            c.dialog_title = _("Not all selected cases deleted")
            c.dialog_text = (
                ungettext(u"%(failures)d of the selected cases could not be"
                          u" deleted.",
                          u"%(failures)d of the selected cases could not"
                          u" be deleted.",
                          failures) % dict(failures=failures)
                + " " + _("Please click OK to continue."))
            template = '/casebundle/dialogs/failure_delete_admin.mako'
        else:
            c.dialog_title = _(u"No Case deleted")
            c.dialog_text = _(u"None of the selected cases could be deleted."
                              u" Please click OK to continue.")

            template = '/casebundle/dialogs/failure_delete_admin.mako'

        if case_bundle.case_list:
            last_id = case_bundle.case_list[-1]
            c.url_ok = url(controller='case_overview',
                           action='index', anchor=last_id)
        else:
            c.url_ok = url(controller='case_overview', action='index')
        if session.get('casebundle'):
           del session['casebundle']
        return render(template)

    # ANONYMIZE
    @checkRole(('cm'))
    def markAnonymize(self, confirmed):
        check_feature('case-module', 'anonymize')
        confirmed = self._checkBool(confirmed)
        c.context = "../main.mako"
        case_bundle = self._get_bundle_or_redirect()
        total = case_bundle.numDatasets()

        if not confirmed:
            c.dialog_title = ungettext(u'Clear case for anonymisation?',
                                       u'Clear cases for anonymisation?',
                                       total)
            c.dialog_text = (
                ungettext(u"Do you really want to clear the following case for"
                          u" anonymisation? The case will not be available to"
                          u" you to work on it and it will be presented to the"
                          u" administrators for review for the definite"
                          u" anonymisation.%s",
                          u"Do you really want to clear the following cases for"
                          u" anonymisation? The cases will not be available to"
                          u" you to work on them and they will be presented to"
                          u" the administrators for review for the definite"
                          u" anonymisation.%s",
                          total)
                % "\n".join(self._buildCaseList(case_bundle)))
            c.url_yes = url(controller='case_bundle',
                            action='markAnonymize', confirmed=1)
            last_id = case_bundle.case_list[-1]
            c.url_no = url(controller='case_overview',
                           action='index', anchor=last_id)
            return render('/casebundle/dialogs/confirm_anonymize.mako')

        abort_unless_POST()
        successes = case_bundle.markAnonymize()
        failures = total - successes

        if successes == total:
            c.success_text = (
                ungettext(u"The selected case was successfully marked for"
                          u" anonymisation. The administrators will review it"
                          u" and perform for the final anonymisation.",
                          u"All selected cases were successfully marked for"
                          u" anonymisation. The administrators will review them"
                          u" and perform for the final anonymisation.",
                          total))
            template = self.overview()
        elif successes > 0:
            # successes != total and successes > 0 means total
            # is also greater than 1, so we can always assume the
            # plural when talking about all selected cases
            c.dialog_title = _(u"Not all cases marked for anonymisation!")
            c.dialog_text = (
                ungettext(u"%(failures)d of the selected cases could not be"
                          u" marked for anonymisation."
                          u" The administrators will review the other cases"
                          u" and perform the final anonymisation ",
                          u"%(failures)d of the selected cases could"
                          u" not be marked for anonymisation."
                          u" The administrators will review the other cases"
                          u" and perform the final anonymisation.",
                          failures)
                % dict(failures=failures)
                + " " + _("Please click OK to continue."))
            template = render('/casebundle/dialogs/failure_anonymize.mako')
        else:
            c.dialog_title = ungettext(u"Case not marked for anonymisation.",
                                       u"Cases not marked for anonymisation.",
                                       total)
            c.dialog_text = (ungettext(u"The selected case was not marked for"
                                       u" anonymisation.",
                                       u"None of the selected cases were marked"
                                       u" for anonymisation.",
                                       total)
                             + " " + _("Please click OK to continue."))
            template = render('/casebundle/dialogs/failure_anonymize.mako')

        if case_bundle.case_list:
            last_id = case_bundle.case_list[-1]
            c.url_ok = url(controller='case_overview',
                           action='index', anchor=last_id)
        else:
            c.url_ok = url(controller='case_overview', action='index')
        if session.get('casebundle'):
            del session['casebundle']
        return template

    @checkRole(('admin'))
    def anonymize(self, confirmed):
        # check if feature is enabled or raise execption
        check_feature('case-module', 'anonymize')
        confirmed = self._checkBool(confirmed)
        c.context = "../main.mako"
        case_bundle = self._get_bundle_or_redirect()
        total = case_bundle.numDatasets()

        if not confirmed:
            c.dialog_title = ungettext(u'Anonymise case?', u'Anonymise cases?',
                                       total)
            c.dialog_text = (
                ungettext(u"Do you really want to anonymise the case and"
                          u" irrevocably lose its personal data? The case"
                          u" will not be available for editing anymore."
                          u" The anonymised case will still be considered"
                          u" in the analysis.%s",
                          u"Do you really want to anonymise the cases and"
                          u" irrevocably lose the personal data? The cases"
                          u" will not be available for editing anymore."
                          u" The anonymised cases will still be considered"
                          u" in the analysis.%s",
                          total)
                % "\n".join(self._buildCaseList(case_bundle)))
            c.url_yes = url(controller='case_bundle', action='anonymize',
                            confirmed=1)
            last_id = case_bundle.case_list[-1]
            c.url_no = url(controller='case_overview',
                           action='index', anchor=last_id)
            return render('/casebundle/dialogs/confirm_anonymize.mako')

        abort_unless_POST()
        successes = case_bundle.anonymize()
        failures = total - successes

        if not failures:
            c.dialog_title = ungettext(u"Case anonymised!",
                                       u"Cases anonymised!",
                                       total)
            c.success_text = (
                ungettext(u"The selected case has been anonymised"
                          u" successfully.",
                          u"The selected cases have been anonymised"
                          u" successfully.",
                          total))
            template = self.overview()
        elif successes > 0:
            # successes != total and successes > 0 means total
            # is also greater than 1, so we can always assume the
            # plural when talking about all selected cases
            c.dialog_title = _(u"Not all cases anonymized.")
            c.dialog_text = (
                ungettext(u"%(failures)s of the selected cases could not be"
                          u" anonymised.",
                          u"%(failures)s of the selected cases could"
                          u" not be anonymised.",
                          failures)
                % dict(failures=failures)
                + " " + _("Please click OK to continue."))
            template = render('/casebundle/dialogs/failure_anonymize.mako')
        else:
            c.dialog_title = ungettext(u"Case not anonymized.",
                                       u"Cases not anonymized.",
                                       total)
            c.dialog_text = (ungettext(u"The selected case has not been"
                                       u" anonymized.",
                                       u"None of the selected cases have been"
                                       u" anonymized.",
                                       total)
                             + " " + _("Please click OK to continue."))
            template = render('/casebundle/dialogs/failure_anonymize.mako')

        if case_bundle.case_list:
            last_id = case_bundle.case_list[-1]
            c.url_ok = url(controller='case_overview',
                           action='index', anchor=last_id)
        else:
            c.url_ok = url(controller='case_overview', action='index')
        if session.get('casebundle'):
            del session['casebundle']
        return template

    # RESTORE
    @checkRole(('admin'))
    def restore(self, confirmed):
        confirmed = self._checkBool(confirmed)
        c.context = "../main.mako"
        case_bundle = self._get_bundle_or_redirect()
        if confirmed == 1:
            abort_unless_POST()
            num = case_bundle.restore()
            c.success_text = ungettext(u"%s was successfully restored.",
                                      u"%s were successfully restored.",
                                      num) % num
            del session['casebundle']
            return self.overview()
        else:
            c.dialog_title = ungettext(u'Restore case?', u'Restore cases?',
                                       len(case_bundle.getCases()))
            c.dialog_text = (
                ungettext(u'Do you really want to restore the following case?'
                          u' It will be available again for editing.%s',
                          u'Do you really want to restore the following cases?'
                          u' They will be available again for editing.%s',
                          len(case_bundle.getCases()))
                % "\n".join(self._buildCaseList(case_bundle)))
            c.url_yes = url(controller='case_bundle', action='restore',
                            confirmed=1)
            last_id = case_bundle.case_list[-1]
            c.url_no = url(controller='case_overview',
                           action='index', anchor=last_id)
            return render('/casemanagement/dialogs/confirm_restore.mako')

    # ORGANISATION
    @checkRole(('admin'))
    @restrict_to_POST
    def set_editor(self):
        c.context = "../main.mako"
        case_bundle = self._get_bundle_or_redirect()
        validator = SetBundleEditorForm()
        try:
            form_result = validator.to_python(request.params)
            num = case_bundle.setEditor(form_result.get('editor'))
            user = UserObject(form_result.get('editor'))
            c.success_text = (
                ungettext(u"%s case was successfully transferred to %s %s",
                          u"%s cases were successfully transferred to %s %s",
                          num) % (num, user.first_name, user.last_name))
            del session['casebundle']
            return self.overview()
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            return render('/casebundle/editor.mako')

    @checkRole(('admin', 'cm'))
    @restrict_to_POST
    def set_standin(self):
        c.context = "../main.mako"
        case_bundle = self._get_bundle_or_redirect()
        validator = SetBundleStandinForm()
        try:
            form_result = validator.to_python(request.params)
            to_add = Set()
            to_remove = Set()
            for key, value in form_result.items():
                if not key.startswith("standin_"):
                    continue
                if value.startswith("add_"):
                    to_add.add(int(value.split("_")[1]))
                elif value.startswith("rem_"):
                    to_remove.add(int(value.split("_")[1]))
            num = case_bundle.modifyStandins(to_add, to_remove)

            if num > 0:
                c.success_text = (
                    ungettext(u"%s case was successfully assigned to a "
                              u"substitute.",
                              u"%s cases were successfully assigned to a"
                              u" substitute.", num) % num)
            else:
                if session.get('casebundle'):
                    del session['casebundle']
                return self._empty_caselist()
            if session.get('casebundle'):
                del session['casebundle']

            return self.overview()
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            return render('/casebundle/standin.mako')

    # EXPORT
    def export(self, type="XML"):
        # check if feature is enabled or raise execption
        check_feature('case-module', 'export%s' % type)
        c.context = "../main.mako"
        case_bundle = self._get_bundle_or_redirect()
        # only pages that are not invisible can be selected by the user.
        # For one thing, the user doesn't know about invisible pages and
        # they are always exported by libformed anyway.
        c.pages = [page for page in g.formedTree.findAllByClass(PageNode)
                   if not page.isInvisible()]
        data = {'exporttype': type,
                'exportanonym': '1',
                'mpuls-toggle-all-exportselection': '1',
                'exportselection': [i.getName() for i in c.pages]}
        c.dialog_title = ungettext(u'Export case?', u'Export cases?',
                                   len(case_bundle.getCases()))
        c.dialog_text = "\n".join(self._buildCaseList(case_bundle))
        # logbook export is only possible for XML exports and if the
        # logbook is actually activated in the application.
        c.allow_logbook_export = (g.mpuls_config.is_enabled('case-module',
                                                            'logbook')
                                  and type == "XML")
        c.allow_documents_export = (g.mpuls_config.is_enabled('case-module',
                                                              'documents')
                                    and type == "XML")

        form = render('/casemanagement/dialogs/confirm_export.mako')
        return render_htmlfill(form, defaults=data, auto_insert_errors=False,
                               errors={})

    def exportAction(self):
        validator = ExportSelectorValidator()
        case_bundle = self._get_bundle_or_redirect()
        form_result = validator.to_python(request.params)
        exporttype = form_result.get('exporttype')
        # check if feature is enabled or raise execption
        check_feature('case-module', 'export%s' % exporttype)
        exportanonym = form_result.get('exportanonym')
        exportselection = form_result.get('exportselection')
        num, export = case_bundle.export(exporttype, exportanonym,
                                         exportselection)
        del session['casebundle']
        if exporttype == "XML":
            # XML is always exported as a generator now.
            assert hasattr(export, "next")
            return self._download_generator(export, "fallakten-export",
                                            extension="xml",
                                            mime_type="application/xml")
        elif exporttype == "XLS":
            return self._download(export, "application/msexcel",
                                  extension="xlsx")
        elif exporttype == "EXLS":
            return self._download(export, "application/msexcel",
                                  extension="xlsx")
        elif exporttype == "CSV":
            return self._download(export, "application/zip", extension="zip")

    # FORMLETTER
    def formletter(self, letterid):
        case_bundle = self._get_bundle_or_redirect()
        num = len(case_bundle)
        lettername = _('unknown')
        for l in g.mpuls_config.get('case', 'formletter'):
            if int(l.get('id')) == int(letterid):
                lettername = l.get('name')
        c.dialog_title = ungettext("%d Case was marked for %s",
                                   "%d Cases were marked for %s", num) % \
            (num, lettername)
        c.dialog_text = ungettext(
            "%d case can be exported. Please click on 'Download' to download "
            "the exported data",
            "%d cases can be exported. Please click on 'Download' to download "
            "the exported data", num) % (num)
        c.letterid = letterid
        return render('/casebundle/dialogs/download_formletter.mako')

    def formletterAction(self):
        letter_id = request.params.get('letterid')
        case_bundle = self._get_bundle_or_redirect()
        d, o = case_bundle.formletter(letter_id)
        export = FormletterExport(d, o).export()
        return self._download(export, "application/msexcel", extension="xls")

    def _download(self, *args, **kwargs):
        name = kwargs.get("name", "fallakten-export")
        kwargs["name"] = name
        return super(CaseBundleController, self)._download(*args, **kwargs)

    def _download_generator(self, generator, name, extension, mime_type,
                            charset="utf8"):
        response.headers['Content-Type'] = \
            '%s; charset=%s' % (mime_type, charset)
        response.headers['Content-Disposition'] = \
            'attachment; filename="%s.%s"' % (name, extension)
        return generator


def build_case_list(cases):
    """
    From a list of cases this function builds a html list of overview fields
    of the cases.
    """
    list = ["<ul>"]
    overview_fields = cases[0].get_overview_fields()
    for case in cases:
        list.append("<li>%s</li>" % ", ".join([
            escape(format_object(
                NA(case.get_value(f.name)))) for f in overview_fields]))
    list.append("</ul>")
    return list

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
