# -*- coding: utf-8 -*-

import logging
import datetime

import formencode

from pylons import request, tmpl_context as c
from pylons.controllers.util import abort

from mpulsweb.lib.translation import _

from mpulsweb.lib.base import BaseController, render, render_htmlfill, \
     restrict_to_POST, abort_unless_POST, session
from mpulsweb.lib.security import checkRole
from mpulsweb.lib.helpers import url
from mpulsweb.lib.dialogs import checkPrivacyStatement
from mpulsweb.model.appointment import CaseAppointmentOverview, \
     CaseAppointmentFactory, CaseReminderOverview, MissingAppointmentError
from mpulsweb.lib.formedforms import FormedForm, Override

log = logging.getLogger(__name__)

def trigger_to_timedelta(meta, value):
    if value is None:
        return None
    return datetime.timedelta(value)

class CaseappointmentForm(FormedForm):

    def __init__(self, action):
        super(CaseappointmentForm, self).__init__(
                "caseappointment.xml", "caseappointment", action,
                modification_checks=True,
                overrides=[Override("trigger",
                                    to_form_result=trigger_to_timedelta)])


caseappointment_new_form = CaseappointmentForm("new_action")
caseappointment_edit_form = CaseappointmentForm("edit_action")

class CaseappointmentController(BaseController):

    def _load_case_appointment(self, id):
        try:
            return CaseAppointmentFactory().loadById(id)
        except MissingAppointmentError:
            abort(404)

    def index(self, id):
        return self.overview(id)

    @checkRole(('admin', 'cm'))
    def overview(self, id):
        id = self._checkInt(id)
        self._ensure_session_case(id)
        c.remindlist = CaseReminderOverview(id)
        c.appointmentlist = CaseAppointmentOverview(id)
        c.show_set_done_title = True
        return render('/casemanagement/appointments.mako')

    @checkRole('cm')
    def new(self, id):
        id = self._checkInt(id)
        self._ensure_session_case(id)
        if not session["case"].is_editable_state():
            abort(401)

        instance = caseappointment_new_form.create_instance()
        instance.set_value("case_id", id)
        cd = datetime.datetime.now()
        instance.set_value("start_date", cd.date())
        instance.set_value("start_time", cd.time())
        instance.set_value("type", 0)
        instance.set_value("trigger", 7)

        return self._render_new_form(instance, {})

    def _render_new_form(self, instance, errors):
        c.rendered_form = caseappointment_new_form.render(instance, errors)
        return render("/casemanagement/newAppointment.mako")

    @checkRole('cm')
    def newReminder(self, id):
        id = self._checkInt(id)
        self._ensure_session_case(id)
        if not session["case"].is_editable_state():
            abort(401)

        instance = caseappointment_new_form.create_instance()
        instance.set_value("case_id", id)
        cd = datetime.datetime.now()
        instance.set_value("start_date", cd.date())
        instance.set_value("start_time", cd.time())
        instance.set_value("type", 1)
        instance.set_value("trigger", 7)

        return self._render_new_form(instance, {})

    @checkRole(('cm'))
    @restrict_to_POST
    def new_action(self):
        instance, errors = caseappointment_new_form.validate_request(
                                                                request.params)

        case_id = instance.get("case_id")
        self._ensure_session_case(case_id)
        if not session["case"].is_editable_state():
            abort(401)

        if errors:
            return self._render_new_form(instance, errors)

        form_result = instance.to_form_result()

        # Create appointment
        factory = CaseAppointmentFactory()
        appointment = factory.createNew(case_id)
        appointment.setData(form_result)
        appointment.store()
        c.success_text = (_(u"The appointment was successfully created."))
        return self.overview(case_id)


    @checkRole(('admin', 'cm'))
    @checkPrivacyStatement('showAppointment')
    def show(self, id, confirmed=True):
        # This method has a "confirmed" parameter because of
        # checkPrivacyStatement which requires it.  The default value of
        # confirmed is True because usually this controller is used when
        # the privacy statement nag-screen has already been shown.
        # Also, it was the default value used in the original routing in
        # jmdweb/config/routing.py.
        # FIXME: do we really need the checkPrivacyStatement decorator here?
        app = self._load_case_appointment(self._checkInt(id))
        self._loadCase(app.case_id)
        c.appointment = app
        return render('/casemanagement/showAppointment.mako')


    def _render_edit_form(self, instance, errors):
        c.rendered_form = caseappointment_edit_form.render(instance, errors)
        return render("/casemanagement/editAppointment.mako")

    @checkRole(('cm'))
    def edit(self, id):
        app = self._load_case_appointment(self._checkInt(id))
        self._ensure_session_case(app.case_id)
        if not session["case"].is_editable_state():
            abort(401)


        instance = caseappointment_edit_form.create_instance()
        instance.set_value('start_date', app.start_date.date())
        instance.set_value('start_time', app.start_date.time())
        instance.set_value('end_date', app.end_date.date())
        instance.set_value('end_time', app.end_date.time())
        instance.set_value('title', app.title)
        instance.set_value('id', app.id)
        instance.set_value('case_id', app.case_id)
        instance.set_value('type', app.type)
        instance.set_value('description', app.description)
        instance.set_value('trigger', app.getTriggerDays())
        instance.set_value('erledigt', app.erledigt)

        c.appointment = app
        c.appointment_id = app.id
        return self._render_edit_form(instance, {})

    @checkRole(('cm'))
    @restrict_to_POST
    def edit_action(self):
        instance, errors = caseappointment_edit_form.validate_request(
                                                                request.params)

        case_id = instance.get("case_id")
        self._ensure_session_case(case_id)
        if not session["case"].is_editable_state():
            abort(401)

        appointment_id = instance.get("id")

        if errors:
            c.appointment_id = appointment_id
            return self._render_edit_form(instance, errors)

        form_result = instance.to_form_result()

        appointment = self._load_case_appointment(appointment_id)

        appointment.setData(form_result)
        appointment.store()
        c.success_text = (_(u"The appointment was successfully edited"))
        return self.overview(appointment.case_id)

    def delete(self, id, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        appointment = self._load_case_appointment(id)
        self._ensure_session_case(appointment.case_id)
        if not session["case"].is_editable_state():
            abort(401)

        if confirmed == 1:
            abort_unless_POST()
            appointment.delete()
            c.success_text = (_(u"The appointment was successfully deleted."))
            return self.overview(appointment.case_id)

        else:
            c.context = "../main.mako"
            c.confirm_for = _(u'Delete appointment?')
            c.question = _(u"Do you really want to delete the appointment\n"
                           u"and irrevocably lose all the data?")
            c.url_yes = url(controller='caseappointment', action='delete',
                            id=id, confirmed=1)
            c.url_no = url(controller='caseappointment', action='index',
                           id=appointment.case_id)
            return render('/casemanagement/dialogs/'
                          'confirm_delete_appointment.mako')
