# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Torsten <irlaender@intevation.de>
#

import logging

import psycopg2.extras
from pylons import config

from mpulsweb.lib.base import render, g, session
from mpulsweb.lib.db import db
from mpulsweb.lib.helpers import ensure_unicode, get_TagSetter


FETCH_SETTINGS_SQL = """SELECT bez, wert from ka_konfiguration_tbl_view"""
STORE_SETTINGS_SQL = """\
UPDATE ka_konfiguration_tbl_view SET wert = %(wert)s WHERE bez = %(bez)s"""

log = logging.getLogger(__name__)


class Agency(object):

    def __init__(self, db=None):
        """Initialize the Agency object.

        The optional db parameter can be used to specify a db object to
        use instead of the thread-global request specific object from
        mpulsweb.lib.db which in some applications may stand for the
        wrong database (e.g. the structure part in JMD which does not
        contain the real agency settings used in the application)
        """
        self._conf = None
        self._db = db

    @property
    def conf(self):
        if self._conf is None:
            self._fetchData()
        return self._conf

    def _get_db(self):
        if self._db is not None:
            return self._db
        return db

    def _fetchData(self):
        self._conf = {}

        db = self._get_db()
        with db.cursor(cursor_factory=psycopg2.extras.DictCursor) as cur:
            cur.execute(FETCH_SETTINGS_SQL)
            rows = cur.fetchall()

            # Manually convert on/off-values to boolean so they can be passed
            # to htmlfill
            for row in rows:
                if row['wert'] == 'on':
                    wert = True
                elif row['wert'] == 'off':
                    wert = False
                elif row['wert'] is None:
                    wert = None
                else:
                    wert = ensure_unicode(row['wert'])
                self._conf[row['bez']] = wert

    def store(self, formdata):
        with self._get_db().transaction() as cur:
            for key, value in formdata.iteritems():
                # Ignore remindercondtion. It is not meant to be editable
                # by the user so ignore it
                if key in ['remindercondition', 'reminderconditiondesc']:
                    continue

                cur.execute(STORE_SETTINGS_SQL, {'bez': key, 'wert': value})

        if g.mpuls_config.is_enabled('case-module', 'checktags'):
            setter = get_TagSetter(session['USER_AUTHORIZED'], config)
            setter.perform()

    def isAnonTransfer(self):
        return self.get_value('anon_transfer', True)

    def getFKZ(self):
        return self.get_value('fkz', u'')

    def getName(self):
        return self.get_value('ka_name', u'')

    def getAddress(self):
        return self.get_value('ka_adresse', u'')

    def getMaxSavetime(self):
        return self.get_value('max_speicherdauer', u'')

    def getReminderCondition(self):
        default = g.mpuls_config.get('case', 'remindercondition')
        return self.get_value('remindercondition', '') or default

    def getReminderConditionDesc(self):
        default = g.mpuls_config.get('case', 'reminderconditiondesc')
        return self.get_value('reminderconditiondesc', '') or default

    def getPrivacyStatement(self):
        form = self.get_value('ee_formular', '')
        if len(form) > 0:
            return form
        return render('/privacy/default/privacy_statement.html')

    def get_value(self, name, default=None):
        return self.conf.get(name, default)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8:
