# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
#

import logging
from datetime import datetime, timedelta
import psycopg2.extras

import mpulsweb.lib.helpers as h
from mpulsweb.lib.base import session, g
from mpulsweb.lib.db import db
from mpulsweb.lib.translation import _, ungettext
from mpulsweb.lib.exceptions import MpulsMissingItemException
import mpulsweb.lib.searchexpr as S
from mpulsweb.model.case import MpulsCaseOverview, ExtendedCaseOverview
from mpulsweb.model.agencysettings import Agency

log = logging.getLogger(__name__)


# Appointment/Reminder types
#
# The first two, APPOINTMENT_NORMAL and APPOINTMENT_REMINDER, are used
# in the database. Applications may define their own types for automatic
# reminders
#
# normal appointment
APPOINTMENT_NORMAL = 0
# normal Reminder
APPOINTMENT_REMINDER = 1
# automatic Reminders generated for tags. This is not rally used
# anymore
APPOINTMENT_TAGS = 2
# Sytem reminder for maximum save time
APPOINTMENT_MAX_SAVETIME = 3
# First value available for application specific appointments
APPOINTMENT_APPLICATION = 100


class MissingAppointmentError(MpulsMissingItemException):

    """Indicates that an apppointment is missing.

    When this exception is raised the appointment that was to be loaded
    does not exist or the user is not allowed to see that appointment.
    """


class AppointmentFactory(object):

    def loadById(self, id):
        with db.cursor() as cur:
            cur.execute(self.get_sql, {'id': id})
            row = cur.fetchone()
            if row is not None:
                return self.createByDBRow(row)
            else:
                raise MissingAppointmentError("%s: no appointment found with"
                                              " ID %r"
                                              % (self.__class__.__name__,
                                                 id))

    def createByData(self, id=None, title=None, sdate=None, edate=None,
                     desc=None, case_id=None, type=0, case_subject=None,
                     trigger=None, erledigt=None, phase=None, editor_name=None):
        appointment = self.constructor()
        appointment.id = id
        appointment.start_date = sdate
        appointment.end_date = edate
        appointment.title = title
        appointment.description = desc
        appointment.case_id = case_id
        appointment.type = type
        appointment.case_subject = case_subject
        appointment.trigger = trigger
        appointment.erledigt = erledigt
        appointment.phase = phase
        appointment.editor_name = editor_name
        return appointment

    def createByDBRow(self, row, with_case_subject=False):
        id = row[0]
        master_id = row[1]
        sdate = row[2]
        edate = row[3]
        name = h.ensure_unicode(row[4])
        desc = h.ensure_unicode(row[5])
        type = row[6]
        trigger = row[7]
        erledigt = row[8]
        case_subject = None
        if with_case_subject:
            case_subject = self.get_case_subject_from_row(row)
        return self.createByData(id, name, sdate, edate, desc, master_id,
                                 type, case_subject, trigger=trigger,
                                 erledigt=erledigt)

    def createNew(self, case_id=None):
        with db.cursor() as cur:
            cur.execute(self.create_sql,
                        {'id': case_id} if case_id is not None else {})
            return self.createByData(cur.fetchone()[0])

    def get_case_subject_from_row(self, row):
        result = {}
        result.update(row)
        result["id"] = result['master_id']
        result["s_id"] = None
        result["status"] = None
        result["zugriff"] = None
        result["phase"] = None
        result["tags"] = None
        result["editor_id"] = None
        result["bearbeiter_id"] = None
        result["changed_editor"] = None
        return g.case_factory.loadFromQueryResult(result).get_subject()


class CaseAppointmentFactory(AppointmentFactory):

    GET_CASE_APPOINTMENT_SQL = """
    SELECT
    id, master_id, sdatum, edatum, name,
    beschreibung, art, trigger, erledigt
    FROM ka_fall_termine_tbl_view
    WHERE id = %(id)s """
    CREATE_CASE_APPOINTMENT_SQL = "SELECT create_fall_termin_ds(%(id)s)"

    def __init__(self):
        self.get_sql = self.GET_CASE_APPOINTMENT_SQL
        self.create_sql = self.CREATE_CASE_APPOINTMENT_SQL
        self.constructor = CaseAppointment


class CaseReminderFactory(AppointmentFactory):

    GET_CASE_APPOINTMENT_SQL = """
    SELECT
    id, master_id, sdatum, edatum, name,
    beschreibung, art, trigger, erledigt
    FROM ka_fall_termine_tbl_view
    WHERE id = %(id)s"""
    CREATE_CASE_APPOINTMENT_SQL = "SELECT create_fall_termin_ds(%(id)s)"

    def __init__(self):
        self.get_sql = self.GET_CASE_APPOINTMENT_SQL
        self.create_sql = self.CREATE_CASE_APPOINTMENT_SQL
        self.constructor = CaseReminder


class GlobalAppointmentFactory(AppointmentFactory):

    GET_GLOBAL_APPOINTMENT_SQL = """
    SELECT
    id, master_id, sdatum, edatum,
    name, beschreibung, art, trigger, erledigt
    FROM ka_global_termine_tbl_view
    WHERE id = %(id)s"""
    CREATE_GLOBAL_APPOINTMENT_SQL = "SELECT create_global_termin_ds()"

    def __init__(self):
        self.get_sql = self.GET_GLOBAL_APPOINTMENT_SQL
        self.create_sql = self.CREATE_GLOBAL_APPOINTMENT_SQL
        self.constructor = GlobalAppointment


class AppointmentOverview(object):

    APPOINTMENT_COLUMNS = """a.id, a.master_id, a.sdatum,
                            a.edatum, a.name, a.beschreibung, a.art,
                            a.trigger, a.erledigt"""

    def _load(self, factory, sql, values, with_case_subject=False):
        with db.cursor(cursor_factory=psycopg2.extras.DictCursor) as cur:
            cur.execute(sql, values)
            self.appointment_list = \
                [factory.createByDBRow(row, with_case_subject=with_case_subject)
                 for row in cur.fetchall()]

    def getDatasets(self):
        return self.appointment_list

    def setDatasets(self, list):
        self.appointment_list = list

    def join(self, overview):
        self.appointment_list.extend(overview.getDatasets())

    def sort(self, cmp=None):
        self.appointment_list = sorted(self.appointment_list)


def trigger_condition(use_trigger):
    if use_trigger:
        return """
            (a.trigger IS NULL
             OR a.sdatum::date - a.trigger <= now()::date )"""
    return "TRUE"


def done_check_condition(use_done_check):
    if use_done_check:
        return "NOT a.erledigt"
    return "TRUE"


class CaseAppointmentOverview(AppointmentOverview):

    GET_CASE_APPOINTMENTLIST_SQL = """
    SELECT """ + AppointmentOverview.APPOINTMENT_COLUMNS + """
    FROM ka_fall_termine_tbl_view a
    WHERE master_id = %(id)s
    AND coalesce(edatum, sdatum)::date+1 >= now()::date
    AND art = 0 {extrawhere}
    ORDER BY sdatum"""

    GET_ALLCASES_APPOINTMENTLIST_SQL = """
    SELECT """ + AppointmentOverview.APPOINTMENT_COLUMNS + """,
    {casefields}
    FROM ka_fall_termine_tbl_view a
    JOIN master_tbl_view m ON m.id = a.master_id
    JOIN ka_status_tbl_view s ON m.id = s.master_id
    WHERE coalesce(edatum, sdatum)::date+1 >= now()::date
    AND art = 0
    AND m.bearbeiter_id = %(editor)s
    AND s.status = 1 {extrawhere}
    ORDER BY sdatum"""

    def __init__(self, caseid=None, use_trigger=False, use_done_check=False):
        afactory = CaseAppointmentFactory()

        extrawhere = "AND " + " AND ".join([
            trigger_condition(use_trigger),
            done_check_condition(use_done_check)])

        if caseid:
            values = {'id': caseid}
            sql = self.GET_CASE_APPOINTMENTLIST_SQL.format(
                extrawhere=extrawhere)
            with_case_subject = False
        else:
            user = session['USER_AUTHORIZED']
            retrieve_fields = g.case_factory.get_search_retrieve_fields()
            retrieve_fields_str = ", ".join([
                ("m.%s") % field for field in retrieve_fields])
            sql = self.GET_ALLCASES_APPOINTMENTLIST_SQL.format(
                casefields=retrieve_fields_str, extrawhere=extrawhere)
            values = {'editor': user.id}
            with_case_subject = True

        self._load(afactory, sql, values, with_case_subject=with_case_subject)


class GlobalAppointmentOverview(AppointmentOverview):

    GET_GLOBAL_APPOINTMENTLIST_SQL = """SELECT
    """ + AppointmentOverview.APPOINTMENT_COLUMNS + """
    FROM ka_global_termine_tbl_view a
    WHERE coalesce(edatum, sdatum)::date+1 >= now()::date {extrawhere}
    ORDER BY sdatum"""

    def __init__(self, use_trigger=False, use_done_check=False):
        afactory = GlobalAppointmentFactory()
        extrawhere = "AND " + " AND ".join([
            trigger_condition(use_trigger),
            done_check_condition(use_done_check)])

        sql = self.GET_GLOBAL_APPOINTMENTLIST_SQL.format(
            extrawhere=extrawhere)
        self._load(afactory, sql, {})


class CaseReminderOverview(AppointmentOverview):

    GET_CASE_REMINDERLIST_SQL = """
    SELECT """ + AppointmentOverview.APPOINTMENT_COLUMNS + """
    FROM ka_fall_termine_tbl_view a
    WHERE master_id = %(id)s {extrawhere}
    AND art = 1
    ORDER BY sdatum"""

    GET_ALLCASES_REMINDERLIST_SQL = """
    SELECT """ + AppointmentOverview.APPOINTMENT_COLUMNS + """,
    {casefields}
    FROM ka_fall_termine_tbl_view a
    JOIN master_tbl_view m ON m.id = a.master_id
    JOIN ka_status_tbl_view s ON m.id = s.master_id
    WHERE art = 1
    AND m.bearbeiter_id = %(editor)s
    AND s.status = 1 {extrawhere}
    ORDER BY sdatum"""

    def __init__(self, caseid=None, use_trigger=False, use_done_check=False):
        afactory = CaseReminderFactory()
        extrawhere = "AND " + " AND ".join([
            trigger_condition(use_trigger),
            done_check_condition(use_done_check)])
        if caseid:
            values = {'id': caseid}
            sql = self.GET_CASE_REMINDERLIST_SQL.format(extrawhere=extrawhere)
            with_case_subject = False
        else:
            user = session['USER_AUTHORIZED']
            retrieve_fields = g.case_factory.get_search_retrieve_fields()
            rfstr = ", ".join([("m.%s") % field for field in retrieve_fields])
            values = {'editor': user.id}
            sql = self.GET_ALLCASES_REMINDERLIST_SQL.format(
                casefields=rfstr, extrawhere=extrawhere)
            with_case_subject = True
        self._load(afactory, sql, values, with_case_subject=with_case_subject)


class Appointment(object):

    def __init__(self):
        self.id = None
        self.title = None
        self.start_date = None
        self.end_date = None
        self.description = None
        self.case_id = None
        # Available types, see APPOINTMENT_* constants
        self.type = APPOINTMENT_NORMAL
        self.case_subject = None
        # trigger is a duration. At start_date - duration triggers the event.
        # e.g. is shown at the black board.
        self.trigger = timedelta(7)
        self.erledigt = False

    def __cmp__(self, other):
        return cmp(self.start_date, other.start_date)

    def _convert2Datetime(self, mdate, mtime, ddate=None, dtime=None):
        log.debug("%s, %s" % (mtime, type(mtime)))
        try:
            date, time = (False, False)
            try:
                y = mdate.year
                m = mdate.month
                d = mdate.day
                date = True
            except Exception:
                date = False

            try:
                H = mtime.hour
                M = mtime.minute
                S = mtime.second
                time = True
            except Exception:
                time = False

            if date and time:
                return datetime(y, m, d, H, M, S)
            elif date:
                H = dtime.hour
                M = dtime.minute
                S = dtime.second
                return datetime(y, m, d, H, M, S)
            elif time:
                y = ddate.year
                m = ddate.month
                d = ddate.day
                return datetime(y, m, d, H, M, S)

            return None

        except Exception, e:
            log.error(e)
            return None

    def getStartDate(self):
        return self.start_date

    def getEndDate(self):
        return self.end_date

    def getDuration(self, empty=''):
        if self.end_date:
            return h.human_timedelta(self.end_date - self.start_date)
        return empty

    def getTrigger(self):
        return self.trigger

    def getTriggerDays(self):
        if self.trigger is not None:
            return self.trigger.days
        return None

    def setData(self, data):
        """Sets values of the appointment to data.
        Data is a dict returned from formencode
        """
        self.title = data.get('title', self.title)
        self.type = data.get('type', self.type)

        self.start_date = self._convert2Datetime(data.get('start_date'),
                                                 data.get('start_time'))

        end_date = self._convert2Datetime(data.get('end_date'),
                                          data.get('end_time'),
                                          data.get('start_date'),
                                          data.get('start_time'))

        # If convert of enddate fail
        if not end_date:
            self.end_date = self.start_date + timedelta(hours=24)
        else:
            self.end_date = end_date

        self.description = data.get('description', self.description)
        self.case_id = data.get('case_id', self.case_id)

        self.trigger = data.get('trigger')
        self.erledigt = data.get('erledigt', False)

    def store(self):
        try:
            fields = {'id': self.id, 'case_id': self.case_id,
                      'type': self.type,
                      'title': self.title, 'description': self.description,
                      'start_date': self.start_date, 'end_date': self.end_date,
                      'trigger': self.trigger, 'erledigt': self.erledigt
                      }
            db.execute(self.store_sql, fields)
            return True
        except Exception:
            log.exception("Error while storing appointment %r for case %r"
                          % (self.id, self.case_id))
        return False

    def isGlobal(self):
        return self.case_id is None

    def isReminder(self):
        return self.type == 1

    def delete(self):
        db.execute(self.delete_sql, {'id': self.id})
        return True

    def getCaseSubject(self):
        if self.case_subject is not None:
            return self.case_subject
        if self.case_id is not None:
            case = g.case_factory.loadById(self.case_id)
            return case.get_subject()
        return ""


class GlobalAppointment(Appointment):

    DELETE_GLOBAL_APPOINTMENT_SQL = "SELECT delete_global_termin_ds(%(id)s)"
    STORE_GLOBAL_APPOINTMENT_SQL = """
    UPDATE ka_global_termine_tbl_view
    SET sdatum = %(start_date)s::timestamp,
    edatum = %(end_date)s::timestamp,
    name = %(title)s,
    beschreibung = %(description)s,
    art = %(type)s,
    trigger = %(trigger)s,
    erledigt = %(erledigt)s
    WHERE id = %(id)s"""

    def __init__(self):
        Appointment.__init__(self)
        self.type = 0
        self.store_sql = self.STORE_GLOBAL_APPOINTMENT_SQL
        self.delete_sql = self.DELETE_GLOBAL_APPOINTMENT_SQL


class CaseAppointment(Appointment):

    DELETE_CASE_APPOINTMENT_SQL = "SELECT delete_fall_termin_ds(%(id)s)"
    STORE_CASE_APPOINTMENT_SQL = """
    UPDATE ka_fall_termine_tbl_view
    SET sdatum = %(start_date)s::timestamp,
    edatum = %(end_date)s::timestamp,
    name = %(title)s,
    beschreibung = %(description)s,
    art = %(type)s,
    trigger = %(trigger)s,
    erledigt = %(erledigt)s
    WHERE id = %(id)s"""

    def __init__(self):
        Appointment.__init__(self)
        self.type = 0
        self.store_sql = self.STORE_CASE_APPOINTMENT_SQL
        self.delete_sql = self.DELETE_CASE_APPOINTMENT_SQL


class CaseReminder(CaseAppointment):

    def __init__(self):
        CaseAppointment.__init__(self)
        self.type = 1

# Tags


class TagsReminderOverview(AppointmentOverview):

    GET_CASES_WITH_TAGS = """
    SELECT m.id
    FROM master_tbl_view m
    JOIN ka_status_tbl_view s ON s.master_id = m.id
    JOIN nm_tags_cases_view t ON t.case_id = m.id
    WHERE s.status = 1
    AND m.bearbeiter_id = %(editor)s
    AND t.tag_id = %(tags)s
    """

    GET_CASES_WITH_TAGS_ADMIN = """
    SELECT m.id
    FROM master_tbl_view m
    JOIN ka_status_tbl_view s ON s.master_id = m.id
    JOIN nm_tags_cases_view t ON t.case_id = m.id
    WHERE s.status = 1
    AND t.tag_id = %(tags)s
    """

    def __init__(self):
        self.appointment_list = []
        user = session['USER_AUTHORIZED']
        reminder_tags = g.mpuls_config.get('overview_tags', 'tag_ids')

        # Return if there are no "important" tags defined
        if not reminder_tags:
            return

        cases = MpulsCaseOverview()

        # 1. Load the user's cases so we can check their tags
        sm = g.case_factory.get_state_machine()
        cases.search({"state": [1, 2, 3], "phase": sm.get_search_item_ids()})

        h.tag_appointment = True

        # Make appointments for all cases that have the "important"-tags
        for tag in reminder_tags:
            case_list = {}
            with db.cursor() as cur:
                if user.isAdmin():
                    cur.execute(self.GET_CASES_WITH_TAGS_ADMIN,
                                {'tags': int(tag)})
                else:
                    cur.execute(self.GET_CASES_WITH_TAGS,
                                {'editor': self.getUserId(), 'tags': int(tag)})
                result = cur.fetchall()

                for r in result:
                    case_list[r[0]] = {}

            cases_with_tags_set = set(case_list)
            cases_with_tags = [case for case in cases.getDatasets()
                               if case.id in cases_with_tags_set]

            log.debug('cases with tags: %s' % case_list)
            for case in cases_with_tags:
                appointment = self.makeAppointment(case,
                                                   g.taggingConf.get_desc(tag),
                                                   g.taggingConf.get_name(tag))
                self.appointment_list.append(appointment)

    def makeAppointment(self, case, description, title):
        factory = CaseAppointmentFactory()
        return factory.createByData(None, title, None, None,
                                    description, case.id, APPOINTMENT_TAGS)

    def getUserId(self):
        """
        Return the User-ID. This function can be overwritten by applications
        that have a "structure" part where the user that owns the cases is not
        the user logged in.
        """
        user = session['USER_AUTHORIZED']
        return user.id


# MAXSAVETIME

class MaxSaveTimeReminderOverview(AppointmentOverview):

    QUERY_TEMPLATE = """
    SELECT m.id, s.zugriff, getLastCaseAppointment(m.id) as last_date,
      m.phase, u.vorname as first_name, u.nachname as last_name
      FROM master_tbl_view m
      JOIN ka_status_tbl_view s ON s.master_id = m.id
      JOIN ka_benutzer_tbl_view u ON u.id = m.bearbeiter_id
     WHERE {CONDITION}
     """

    def __init__(self, case=None):
        user = session['USER_AUTHORIZED']
        sm = g.case_factory.get_state_machine()

        # create reminders
        agency = Agency()
        maxage = agency.getMaxSavetime()
        stale_info = self.search_cases(agency, sm.get_active_state_ids(),
                                       case=case)
        overdue_info = self.search_cases(agency, sm.get_end_state_ids(),
                                         case=case)

        search_phases = sm.get_search_item_ids()

        if not stale_info and not overdue_info:
            # In the case where no reminders are found we don't need to
            # load any cases.
            cases = []
        elif case is not None:
            # There must be exactly one potential case with a reminder:
            # case_ids is true here and a case was given as argument.
            # That case must be the one found.
            assert set([case.id]) == set(stale_info) | set(overdue_info)

            # The case only actually has a reminder if it's in the right
            # state and phase
            state = case.getState()
            if (state.getState() in [1, 2, 3]
                and state.getPhase() in search_phases):
                cases = [case]
            else:
                cases = []
        else:
            # Load the user's cases so we can check the maximum savetime
            # on them
            search = ExtendedCaseOverview()
            search.search(S.And([S.Status([1, 2, 3]),
                                 S.Choice("phase", sm.get_search_item_ids()),
                                 S.Choice("id", list(set(stale_info)
                                                     | set(overdue_info)))]))
            cases = search.getDatasets()

        stale_reminders = self.create_appointments(cases, stale_info,
                                                   "last_date", maxage,
                                                   finished=False)
        overdue_reminders = self.create_appointments(cases, overdue_info,
                                                     "access", maxage,
                                                     finished=True)
        self.appointment_list = stale_reminders + overdue_reminders

    def search_cases(self, agency, phases, case):
        # Load ids and some additional info on cases which either are
        # active but seems to be orphaned or are finished and not edited
        # for longer than maxage.
        # orphaned means that all last appointments, aids or the
        # modification date is older than maxage
        user = session['USER_AUTHORIZED']

        conditions = ["s.status <> ALL (%(forbidden_statuses)s)",
                      "extract(day from now() - s.zugriff) > %(maxdays)s",
                      "phase in %(phases)s",
                      agency.getReminderCondition(),
                      ]
        params = {"forbidden_statuses": [5] if user.isAdmin() else [3,4,5],
                  "maxdays": agency.getMaxSavetime(),
                  "phases": tuple(phases),
                  }
        if not user.isAdmin():
            conditions.append("m.bearbeiter_id = %(editor)s")
            params["editor"] = user.id
        if case is not None:
            conditions.append("m.id = %(case_id)s")
            params["case_id"] = case.id

        query = self.QUERY_TEMPLATE.format(CONDITION=" AND ".join(conditions))

        with db.cursor() as cur:
            cur.execute(query, params)
            return dict((row[0], {'access': row[1], 'last_date': row[2],
                'phase': row[3],
                'editor_name': u'%s, %s' % (h.ensure_unicode(row[5]),
                                            h.ensure_unicode(row[4])[:1]) })
                         for row in cur.fetchall())

    def create_appointments(self, cases, case_info, info_key, maxage, finished):
        appointments = []
        for case in cases:
            info = case_info.get(case.id)
            if info is None:
                continue
            appointments.append(self.makeAppointment(case, info[info_key],
                                                     maxage, finished=finished,
                                                     phase=info['phase'],
                                                     editor_name=\
                                                            info['editor_name']
                                                     ))
        return appointments

    def makeAppointment(self, case, last_date, maxage, finished,
            phase=None, editor_name=None):
        days_overdue = (datetime.now() - last_date).days - int(maxage)
        if finished:
            title = _(u"Max save duration exceeded!")
            desc = ungettext('Attention! The maximum save duration is '
                             'exceeded for %s day for this case document!',
                             'Attention! The maximum save duration is '
                             'exceeded for %s days for this case document!',
                             days_overdue) % (days_overdue,)
        else:
            title = _(u"Inactive case document")
            desc = _(u"Inactive case document? Please check,"
                     u" if the case will still be edited.")

        factory = CaseAppointmentFactory()
        return factory.createByData(None, title,
                                    datetime.now() - timedelta(days_overdue),
                                    None, desc, case.id,
                                    APPOINTMENT_MAX_SAVETIME,
                                    case.get_subject(),
                                    phase=phase, editor_name=editor_name)


class NextAppointmentOverview(AppointmentOverview):

    GET_APPOINTMENTLIST_SQL = """
    SELECT """ + AppointmentOverview.APPOINTMENT_COLUMNS + """
    FROM ka_fall_termine_tbl_view a
    WHERE master_id = %(id)s
    AND sdatum::date >= now()::date
    AND ( art = 0 OR art = 1) {extrawhere}
    ORDER BY sdatum LIMIT 1"""

    def __init__(self, caseid):
        afactory = CaseAppointmentFactory()
        extrawhere = "AND " + done_check_condition(True)
        values = {'id': caseid}
        sql = self.GET_APPOINTMENTLIST_SQL.format(extrawhere=extrawhere)
        self._load(afactory, sql, values)

    def get_next_appointment(self):
        if self.getDatasets():
            return self.getDatasets()[0]
        return None


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8:
