import unittest
import datetime

from mpulsweb.lib.searchexpr import And, Field, TRUE, Between, to_searchtree, \
                                    to_simple_format_from_json, Choice, \
                                    Responsibilities
import mpulsweb.lib.sqlexpr as E


class TestToSimpleFormat(unittest.TestCase):

    def test_TRUE(self):
        t = TRUE
        self.assertEquals(t.to_simple_format()['cls'], 'TRUE')
        self.assertEquals(t.to_simple_format()['attrs'], {})

    def test_Field(self):
        f = Field("test", 12)
        sf = f.to_simple_format()
        self.assertEquals(sf['cls'], 'Field')
        self.assertEquals(sf['attrs']['value'], 12)
        self.assertEquals(sf['attrs']['fieldname'], 'test')
        self.assertEquals(sf['attrs']['op'], 'EQ')

    def test_Field_operator(self):
        for op in ('LT', 'LE', 'GT', 'GE', 'Substring', 'Startwith', 'NE'):
            f = Field("test", 12, getattr(E, op))
            sf = f.to_simple_format()
            self.assertEquals(sf['attrs']['op'], op)

    def test_Between(self):
        b = Between("test", 1, 12)
        sf = b.to_simple_format()
        self.assertEquals(sf['cls'], 'Between')
        self.assertEquals(sf['attrs']['low'], 1)
        self.assertEquals(sf['attrs']['fieldname'], 'test')
        self.assertEquals(sf['attrs']['high'], 12)

    def test_Between_date(self):
        b = Between("test",
                    datetime.date(2013, 2, 1), datetime.date(2014, 7, 31))
        self.assertEquals(b.to_simple_format(),
                          {'cls': 'Between',
                           'attrs': {'fieldname': 'test',
                                     'high': datetime.date(2014, 7, 31),
                                     'low': datetime.date(2013, 2, 1)},
                           })

    def test_And(self):
        test1 = And([TRUE, TRUE,
                     And([Field('test', 'XXX'), Between('test', 1, 3)])])
        sf = test1.to_simple_format()
        self.assertEquals(sf['cls'], 'And')
        self.assertEquals(sf['attrs']['exprs'][0]['cls'], 'TRUE')
        self.assertEquals(sf['attrs']['exprs'][1]['cls'], 'TRUE')
        self.assertEquals(sf['attrs']['exprs'][2]['cls'], 'And')
        self.assertEquals(sf['attrs']['exprs'][2]['attrs']['exprs'][1]['cls'],
                          'Between')

        test2 = And([TRUE, TRUE])
        sf = test2.to_simple_format()
        self.assertEquals(sf['cls'], 'And')
        self.assertEquals(sf['attrs']['exprs'][0]['cls'], 'TRUE')
        self.assertEquals(sf['attrs']['exprs'][1]['cls'], 'TRUE')

    def test_Choice(self):
        choice = Choice("test", [1, 2, 3])
        sf = choice.to_simple_format()
        self.assertEquals(sf['cls'], 'Choice')
        self.assertEquals(sf['attrs']['fieldname'], 'test')
        self.assertEquals(sf['attrs']['values'], [1, 2, 3])


class TestToSearchTree(unittest.TestCase):

    def test_tost_TRUE(self):
        true1 = to_searchtree({'cls': 'TRUE', 'attr': {}})
        self.assertEquals(true1, TRUE)

    def test_tost_And(self):
        and1 = to_searchtree({'cls': 'And',
                              'attrs': {'exprs':[{'cls': 'TRUE',
                                                  'attrs': {}}]}})
        self.assertEquals(and1.__class__, And)
        self.assertEquals(and1.exprs[0], TRUE)

    def test_tost_Field(self):
        field1 = to_searchtree({'cls': 'Field',
                                'attrs': {'fieldname':'test',
                                          'op': 'EQ', 'value':1}})
        self.assertEquals(field1.__class__, Field)
        self.assertEquals(field1.fieldname, 'test')
        self.assertEquals(field1.value, 1)

    def test_tost_Field_operator(self):
        for op in ('LT', 'EQ', 'LE', 'GT', 'GE',
                   'Substring', 'Startwith', 'NE'):
            field1 = to_searchtree({'cls':'Field',
                                    'attrs': {'fieldname':'test',
                                              'op': op, 'value':1}})
            self.assertTrue(field1.op == getattr(E, op))

    def test_tost_Between(self):
        between1 = to_searchtree({'cls': 'Between',
                                  'attrs':{'fieldname':'test',
                                           'low': 1, 'high':5}})
        self.assertEquals(between1.__class__, Between)
        self.assertEquals(between1.fieldname, 'test')
        self.assertEquals(between1.low, 1)
        self.assertEquals(between1.high, 5)


class TestToSearchTreeCompatibility(unittest.TestCase):

    def decode_json(self, jsonstring):
        return to_searchtree(to_simple_format_from_json(jsonstring))

    def test_responsibilities_no_changed_owner(self):
        """Responsibilities did not always have the changed_editor flag.
        """
        andExpr = self.decode_json(
            u"""{"attrs": {
                    "exprs": [
                       {"attrs": {"standin": false, "own": true},
                        "cls": "Responsibilities"}
                    ]
                },
                "cls": "And"}""")
        expr = andExpr.exprs[0]
        self.assertIsInstance(expr, Responsibilities)
        self.assertTrue(expr.own)
        self.assertFalse(expr.standin)
        self.assertFalse(expr.changed_editor)

    def test_changed_editor_no_responsibilities(self):
        """Test loading ChangedEditor without a Responsibilities entry.
        """
        andExpr = self.decode_json(
            """{"attrs": {"exprs": [{"attrs": {}, "cls": "ChangedEditor"}]},
                "cls": "And"}""")
        expr = andExpr.exprs[0]
        # ChangedEditor is automatically converted into a
        # Responsibilities instance.
        self.assertIsInstance(expr, Responsibilities)
        self.assertFalse(expr.own)
        self.assertFalse(expr.standin)
        self.assertTrue(expr.changed_editor)

    def test_changed_editor_with_responsibilities(self):
        """Test loading ChangedEditor without a Responsibilities entry.
        """
        andExpr = self.decode_json(
            """{"attrs": {
                   "exprs": [
                       {"attrs": {"standin": false, "own": true},
                        "cls": "Responsibilities"},
                       {"attrs": {}, "cls": "ChangedEditor"}
                   ]
                },
                "cls": "And"}""")
        expr = andExpr.exprs[0]
        # ChangedEditor is automatically merged into the
        # Responsibilities instance.
        self.assertIsInstance(expr, Responsibilities)
        self.assertTrue(expr.own)
        self.assertFalse(expr.standin)
        self.assertTrue(expr.changed_editor)


class TestSearchTreeJson(unittest.TestCase):

    def assertRoundTrip(self, expr):
        roundtripped = to_searchtree(to_simple_format_from_json(expr.to_json()))
        # we don't have a way to directly compare two exprs, but their
        # simple format representations should be equal, too, so we
        # check that.
        self.assertEquals(expr.to_simple_format(),
                          roundtripped.to_simple_format())

    def test_to_json(self):
        self.assertRoundTrip(And([TRUE, Between("test", 1, 2),
                                  Field("test2", 5)]))

    def test_between_dates(self):
        self.maxDiff = None
        self.assertRoundTrip(And([Between("test", datetime.date(2013, 2, 1),
                                          datetime.date(2014, 7, 31))]))


if __name__ == "__main__":
    unittest.main()
