# -*- coding: utf-8 -*-
# This file is part of Kumasta.
#
# Kumasta is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Kumasta is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Kumasta.  If not, see <http://www.gnu.org/licenses/>.

import atexit
import datetime
import glob
import logging
import os.path
import subprocess
import sys
import tempfile
import time
import zipfile
import locale

from xml.sax.saxutils import escape

logger = logging.getLogger(__name__)

SEARCHPATHS = ['/usr/share/pyshared', '/usr/lib/libreoffice/program'] + \
               glob.glob('/usr/lib*/libreoffice*') + \
               glob.glob('/usr/lib*/openoffice*') + \
               glob.glob('/usr/lib*/ooo*') + \
               glob.glob('/opt/libreoffice*') + \
               glob.glob('/opt/openoffice*') + \
               glob.glob('/opt/ooo*') + \
               glob.glob('/usr/local/libreoffice*') + \
               glob.glob('/usr/local/openoffice*') + \
               glob.glob('/usr/local/ooo*') + \
               glob.glob('/usr/local/lib/libreoffice*')

#              glob.glob('/usr/lib/libreoffice/program')
#              glob.glob('/usr/lib*/openoffice*/program/') + \
#              glob.glob('/usr/lib*/ooo*/program')

# soffice can be used for OpenOffice and LibreOffice
OFFICE_EXE = "soffice"
OFFICE_ARG_ACCEPT = "-accept=socket,host=%(host)s,port=%(port)s;urp;StarOffice.ServiceManager"
OFFICE_ARGS = [
    "-norestore",
    "-nofirststartwizard",
    "-nologo",
    "-invisible",
    "-headless"
    ]

META_TEMPLATE=u"""<?xml version="1.0" encoding="UTF-8"?>
<office:document-meta xmlns:office="urn:oasis:names:tc:opendocument:xmlns:office:1.0" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:meta="urn:oasis:names:tc:opendocument:xmlns:meta:1.0" xmlns:ooo="http://openoffice.org/2004/office" xmlns:grddl="http://www.w3.org/2003/g/data-view#" office:version="1.2">
  <office:meta>
    <meta:generator>CHOSOUTIL</meta:generator>
    <dc:title>%(title)s</dc:title>
    <meta:initial-creator>%(initial_creator)s</meta:initial-creator>
    <dc:creator>%(creator)s</dc:creator>
    <meta:creation-date>%(creation_date)s</meta:creation-date>
    <dc:date>%(mod_date)s</dc:date>
    <meta:print-date>%(print_date)s</meta:print-date>
  </office:meta>
</office:document-meta>
"""

def initialize_logging():
    """Initializes the logging system"""
    root = logging.getLogger()
    root.setLevel(logging.ERROR)
    hdlr = logging.StreamHandler()
    fmt = logging.Formatter("%(asctime)s %(levelname)s %(message)s")
    hdlr.setFormatter(fmt)
    root.addHandler(hdlr)

initialize_logging()

for path in SEARCHPATHS:
    try:
        sys.path.append(path)

        # Save and restore locale settings around the import of uno.
        # This is a work-around for a defect in the uno module which
        # apparently always sets the locale to en_US.utf8 (at least on
        # Debian wheezy). See mpuls/issue4026 for details.
        # This solution is fragile because it only works if this is the
        # first time the module is being imported in the python process.
        old_locale = locale.setlocale(locale.LC_ALL)
        try:
            import uno
        finally:
            locale.setlocale(locale.LC_ALL, old_locale)

        import unohelper
        from com.sun.star.connection import NoConnectException
        os.environ['PATH'] = '%s:' % path + os.environ['PATH']
        logger.debug("Using uno found at: %s " % path)
        break
    except ImportError:
        sys.path.remove(path)
        continue
else:
    logger.error("Could not find office installation in: %s " % SEARCHPATHS)

class ChosoException(Exception):
    """
    A generic exception while working with Office documents.
    The Exception can contain additional error output
    """

    def __init__(self, value, output = None):
        Exception.__init__(self, value)
        self.output = output

globalOffice = None
def get_global_office():
    global globalOffice
    if not globalOffice:
        globalOffice = Office()
    return globalOffice

def kill_global_office():
    global globalOffice
    if globalOffice:
        globalOffice.die()

# Make sure we shutdown office when we terminate
atexit.register(kill_global_office)

class Office(object):
    """
    Main Class that communicates and controls withthe Children of Start
    Office instance.

    If there is no Office running at the specified address a new instance
    is started on initialization.
    """

    def __init__(self, host="localhost", port="2003", headless=False):
        self.oopid = 0
        unocontext = None
        self.host = host
        self.port = port
        self.context = uno.getComponentContext()
        resolver = self.context.ServiceManager.createInstanceWithContext(
                "com.sun.star.bridge.UnoUrlResolver", self.context)

        try:
            unocontext = resolver.resolve("uno:socket,host=" + host + ",port="
                                + port + ";urp;StarOffice.ServiceManager" )
        except NoConnectException, e:
            logger.debug("Existing listener not found.\n")
            try:
                newenv = os.environ.copy()
                # set home directory for running in mod_wsgi
                # the user that executes the process needs write access to /tmp
                # and /tmp/..openoffice.org
                newenv["HOME"] = "/tmp"

                self.oopid = os.spawnvpe(os.P_NOWAIT, OFFICE_EXE,
                        [OFFICE_EXE] + OFFICE_ARGS +
                        [OFFICE_ARG_ACCEPT % dict(host=host, port=port)],
                        newenv);
                logger.debug( "executing command: " + " ".join([OFFICE_EXE] +
                        OFFICE_ARGS +
                        [OFFICE_ARG_ACCEPT % dict(host=host, port=port)]))

                waited = 0
                while waited < 10:
                    try:
                        unocontext = resolver.resolve("uno:socket,host=" + host +
                        ",port=" + port + ";urp;StarOffice.ServiceManager")
                        break
                    except NoConnectException, e:
                        time.sleep(1)
                        waited += 1

            except Exception, e:
                logger.error("Launch of %s failed.\n%s" % (OFFICE_EXE, e))
                raise ChosoException("Launch of %s failed." % OFFICE_EXE, e)

        if not unocontext:
            raise ChosoException("Unable to connect or start Office.")

        self.cwd = unohelper.systemPathToFileUrl( os.getcwd() )

    def die(self):
        if self.oopid:
            logger.debug('Shutting down open office withself.oopid: %s' % self.oopid)
            try:
                os.kill(self.oopid, 15)
                state=(0, 0)
                waited = 0
                while (state == (0, 0) and waited < 15):
                    logger.debug('Waiting for OpenOffice with pid %s '
                            'to disappear.' % self.oopid)
                    state = os.waitpid(self.oopid, os.WNOHANG)
                    if (state != (0, 0)):
                        break
                    waited += 1
                    time.sleep(1)
                else:
                    logger.warn("Terminating %s" % self.oopid)
                    os.kill(self.oopid, 9)
            except:
                logger.error('Could not find Process %s' % self.oopid)

    def convert(self, infile, outfile, doctype='document', format='pdf'):
        """
        Utilizes unoconv to convert infile to outfile using our office instance
        """
        command = ["unoconv", "--server", self.host, "--port", self.port,
                   "--doctype", doctype, "--format", format, "--stdout",
                   infile]

        logger.debug("Executing: %s" % command)

        with open(outfile, "wb") as of:
            proc = subprocess.Popen(command, stdout=of,
                                    stderr=subprocess.PIPE)
        _, error = proc.communicate()
        #if proc.returncode != 0:
        #    print proc.returncode
        #    print error
        #    raise ChosoException("Error running: %s, \nError: %s " % (" ".join(command), error))

    def odt_to_doc(self, filename):
        """
        Converts the given file from OpenDocument to Microsoft Office 97
        """

        infile = os.path.abspath(filename)
        outfile = infile.replace('odt', 'doc')
        self.convert(infile, outfile, format="doc")

    def odt_to_pdf(self, filename):
        """
        Converts the given file from OpenDocument to Portable Document Format
        """

        infile = os.path.abspath(filename)
        outfile = infile.replace('odt', 'pdf')

        logger.debug('Source file: file://%s' % infile)
        logger.debug('Target file: %s' % outfile)
        self.convert(infile, outfile, format="pdf")

def read_odt_file(odtfilename):
    """
    Reads the string of the odt file
    """

    zfile = zipfile.ZipFile(odtfilename, "r")
    archive = []
    for name in zfile.namelist():
        archive.append((name, zfile.read(name)))

    zfile.close()
    return archive

def replace_content(archive, changes):
    """
    Replaces the strings from the changes dictionary in the archive contents
    """
    archive2 = []
    for (name, text) in archive:
        # TODO: headers are saved in styles.xml, replacing text there is not
        #       supported yet

        if name == "content.xml":
            text = unicode(text, 'UTF-8')
            for cmark, ctext in changes.items():
                # If you want to force a break do it hard, soft breaks are
                # arbitratily placed in the middle of text
                text = text.replace("<text:soft-page-break/>", "")
                text = text.replace(cmark, ctext)
            text2 = unicode.encode(text,'UTF-8')
        else:
            text2 = text
        archive2.append((name,text2))
    return archive2

def expand_odt(odt, variables):
    """
    Expands template markers in the source odt with values
    from the dictionary variables. Values are converted to a
    unicode representation and escaped.

    @param odt: Filename of the Open Office Document that should be
                used as template
    @param variables: Dictionary with key,value pairs to expand in the odt

    @returns the resulting odt archive
    """
    template_dict = {}
    for key, value in variables.items():
        if value == None:
            value = ""
        if isinstance(value, bool):
            if value:
                value = "X"
            else:
                value = ""
        if isinstance(value, datetime.date):
            value = value.strftime("%d.%m.%Y")
        if not isinstance(value, unicode):
            value = unicode(str(value), 'UTF-8')
        template_dict[key] = escape(value)

    return replace_content(read_odt_file(odt), template_dict)

def set_meta_odt(archive, title=u"", initial_creator=u"chosoutil", creator=u"chosoutil",
        creation_date=None, mod_date=None, print_date=None):
    """
    Sets the document meta data

    @param odt: ODT Archive that should be modifierd
    @param title: The Title of the document
    @param initial_creator: The initial creator setting
    @param creator: The creator of the document
    @param creation_date: The Date of creation. If None, Now is used.
    @param mod_date: The Date of modification. If None, Now is used.
    @param print_date: The Date of printing. If None, Now is used.

    @returns the resulting odt archive
    """
    variables = {
            "title" : title,
            "initial_creator" : initial_creator,
            "creator" : creator,
        }

    timefmt="%Y-%m-%dT%H:%M:00"
    if creation_date:
        if isinstance(creation_date, datetime.datetime):
            variables["creation_date"] = creation_date.strftime(timefmt)
        else:
            variables["creation_date"] = creation_date
    else:
        variables["creation_date"] = datetime.datetime.now().strftime(timefmt)
    if mod_date:
        if isinstance(mod_date, datetime.datetime):
            variables["mod_date"] = mod_date.strftime(timefmt)
        else:
            variables["mod_date"] = mod_date
    else:
        variables["mod_date"] = datetime.datetime.now().strftime(timefmt)
    if print_date:
        if isinstance(print_date, datetime.datetime):
            variables["print_date"] = print_date.strftime(timefmt)
        else:
            variables["print_date"] = print_date
    else:
        variables["print_date"] = datetime.datetime.now().strftime(timefmt)

    archive2 = []
    for (name, text) in archive:
        if name == "meta.xml":
            text = unicode.encode(META_TEMPLATE % variables, 'UTF-8')
        archive2.append((name,text))
    return archive2


def write_odt_file(filename, archive):
    """
    Writes the archive contents to a odt file.

    @param filename: The output filename
    @param archive: The input odt archive
    """
    zfile = zipfile.ZipFile(filename, "w")
    for (name, text) in archive:
        zfile.writestr(name, text)
    zfile.close()

def write_pdf_file(filename, archive):
    """
    Writes the archive contents to a pdf file.

    @param filename: The output filename
    @param archive: The input odt archive
    """
    odtfname = tempfile.mkstemp()[1] + '.odt'
    zfile = zipfile.ZipFile(odtfname, "w")
    for (name, text) in archive:
        zfile.writestr(name, text)
    zfile.close()
    get_global_office().convert(odtfname, filename)
    os.remove(odtfname)

def write_doc_file(filename, archive):
    """
    Writes the archive contents to a doc file.

    @param filename: The output filename
    @param archive: The input odt archive
    """
    odtfname = tempfile.mkstemp()[1] + '.odt'
    zfile = zipfile.ZipFile(odtfname, "w")
    for (name, text) in archive:
        zfile.writestr(name, text)
    zfile.close()
    get_global_office().convert(odtfname, filename, format='doc')
    os.remove(odtfname)

def get_pdf_file(archive):
    """
    Converts an odt archive to pdf and retuns the pdf

    @param archive: The input odt archive
    @returns: The output pdf file
    """

    pdfname = tempfile.mkstemp()[1]
    write_pdf_file(pdfname, archive)
    ret = ""
    with open(pdfname, "r") as pdf:
        ret = pdf.read()
    os.remove(pdfname)
    return ret

def get_doc_file(archive):
    """
    Converts an odt archive to doc and retuns the doc

    @param archive: The input odt archive
    @returns: The output doc file
    """

    docname = tempfile.mkstemp()[1]
    write_doc_file(docname, archive)
    ret = ""
    with open(docname, "r") as doc:
        ret = doc.read()
    os.remove(docname)
    return ret
