# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 by Intevation GmbH
#
# FormEd is free software under the terms of the
# GNU General Public License Version 3. See file
# LICENSE coming with the source of FormEd for details.
#
# Authors:
# Bernhard Herzog     <bernhard.herzog@intevation.de>

"""Generate SQL for insertion-only views of master_tbl.
"""

import codecs
from datetime import datetime
from itertools import chain
from textwrap import dedent

from formed.meta import data
from formed.meta.structure import StructureTree, StructureNode
from formed.meta.data import RepeatNode, TextLeaf, DateLeaf, IntLeaf, ChoiceNode
from formed.meta.nodecomponents import Leaf


class ExportInsertSQL(object):

    def __init__(self, document):
        self.document = document

    @staticmethod
    def getDescription():
        return "Export document as SQL statements for insertion-only views"

    def doExport(self, path):
        tree = StructureTree(self.document)

        # We need to support attachments which are not defined in the
        # formed-trees, so add a fake ka_dokumente repeat group which
        # lets us keep the Exporter class completely generic.
        add_extra_meta_nodes(tree)

        # FIXME: The name of the user which needs access to the views
        # should not be hard-wired, but user-supplied or otherwise
        # defined in some configuration file.
        exporter = Exporter(codecs.open(path, "wb", "UTF-8"),
                            "bewerbungsimport")
        exporter.export_tree(tree)


def add_extra_meta_nodes(tree):
    extra_nodes = extra_meta_nodes()
    old_root = tree.root
    tree.root = StructureNode(old_root.meta_node, old_root.ordered_leaves,
                              old_root.ordered_repeat_groups + extra_nodes)


def extra_meta_nodes():
    # There are no length limits on the text nodes, because mpuls
    # applications differ in the limits used (e.g. default limit of
    # beschreibung is 256, mpuls_S uses 512) and the limits may change
    # over time and whenever a limit changes the values need to be
    # modified here as well. It's easy to forget that (see
    # mpuls/issue5233), and to avoid this, we omit the limits here. The
    # limits are enforced indirectly anyway, because an insert on the
    # import views created here is turned into an insert on the
    # underlying table and the table's limits are enforced at that
    # point.
    return [extra_node("ka_dokumente",
                       [new_node(TextLeaf, "mime"),
                        new_node(TextLeaf, "name"),
                        new_node(IntLeaf, "groesse"),
                        new_node(ByteALeaf, "daten"),
                        new_node(TextLeaf, "beschreibung"),
                        new_node(DateLeaf, "datum"),
                        ]),
            extra_node("ka_logbuch",
                       [new_node(DateTimeLeaf, "datum"),
                        new_node(ChoiceNode, "art"),
                        new_node(ChoiceNode, "typ"),
                        new_node(TextLeaf, "bearbeiter"),
                        new_node(TextLeaf, "notiz"),
                        ]),
            ]


def extra_node(tbl_name, leaves):
    # For the extra nodes, the insert and update triggers must be
    # disabled to avoid updating the last access time in the
    # ka_status_tbl entry associated with the case the documents being
    # added belong to. This should be safe to do for both initial
    # submissions and later attestation submissions even though it is
    # only needed for the latter because for an initial submission
    # master_tbl itself is also modified which should result in an
    # uptodate ka_status_tbl entry.
    def disable_trigger(action):
        return ("ALTER TABLE %(tbl)s_tbl"
                " DISABLE TRIGGER t1_%(tbl)s_tbl_%(action)s;"
                % dict(tbl=tbl_name, action=action))
    def enable_trigger(action):
        return ("ALTER TABLE %(tbl)s_tbl"
                " ENABLE TRIGGER t1_%(tbl)s_tbl_%(action)s;"
                % dict(tbl=tbl_name, action=action))

    node = StructureNode(new_node(RepeatNode, tbl_name), leaves, [])
    node.settings = {"set_up": [disable_trigger("insert"),
                                disable_trigger("update")],
                     "clean_up": [enable_trigger("update"),
                                  enable_trigger("insert")],
                     }
    return node


def new_node(cls, name, **attrs):
    node = cls()
    node.setName(name)
    for key, value in attrs.items():
        node.setAttribute(key, value)
    return node


class ByteALeaf(Leaf):

    """'Fake' Leaf for BLOBs for attachments"""


class DateTimeLeaf(Leaf):

    """'Fake' DateTime leaf type for logbook entries."""


class Field(object):

    def __init__(self, name, sql_type):
        self.name = name
        self.sql_type = sql_type


class Exporter(object):

    def __init__(self, out, username):
        self.out = out
        self.username = username
        self.teardown = []

    def add_teardown(self, cmd):
        self.teardown.append(cmd)

    def write_block(self, text):
        self.out.write(dedent(text))

    def export_tree(self, tree):
        self.export_header(tree.get_version())
        self.export_recursively(tree.root, None)
        self.export_footer()
        self.export_teardown()

    def export_header(self, version):
        self.write_block("""\
        --
        -- Generated by libformed on %s
        -- formedtree: %s
        --

        BEGIN;

        SET ROLE :adm_ka_owner;

        """ % (datetime.now().strftime("%a, %Y-%m-%d %H:%M:%S +0000"), version))

    def export_footer(self):
        self.write_block("""
        COMMIT;
        """)

    def export_teardown(self):
        self.out.write("\n-- Commands to remove the objects created here:\n\n")
        self.out.write("\n-- BEGIN;\n")
        for cmd in reversed(self.teardown):
            self.out.write("-- " + cmd + "\n")
        self.out.write("-- COMMIT;\n")

    def export_recursively(self, node, parent):
        self.export_node(node, parent)
        for rg in node.ordered_repeat_groups:
            self.export_recursively(rg, node)

    def node_basename(self, node):
        return node.name if node.name is not None else "master"

    def export_node(self, node, parent):
        node_name = self.node_basename(node)
        table_name = node_name + "_tbl"

        settings = getattr(node, "settings", {})
        set_up = settings.get("set_up", [])
        clean_up = settings.get("clean_up", [])

        insert_view_name = node_name + "_insert_bewerbung_view"
        insert_rule_name = node_name + "_insert_bewerbung_rule"
        insert_function_name = "insert_bewerbung_" + node_name

        if parent is None:
            fields = []
            special_fields = [
                ("bearbeiter_id",
                 "(select id from ka_benutzer_tbl where login = 'neueingang')"),
                ]
            extra_statements = [
                "PERFORM create_status_ds(rid);",
                ]
        else:
            fields = [Field(self.node_basename(parent) + "_id",
                            "INTEGER")]
            special_fields = []
            extra_statements = []

        fields.extend(self.meta_to_field(meta) for meta in node.ordered_leaves)

        def line(s):
            self.out.write(s + "\n")

        line("CREATE OR REPLACE VIEW %s AS SELECT" % insert_view_name)
        line(",\n".join("    (NULL::%s) AS %s" % (field.sql_type, field.name)
                        for field in fields))
        line("WHERE FALSE;")
        line("")

        self.add_teardown("DROP VIEW %s;" % insert_view_name)

        for l in self.generate_insert_function(insert_function_name,
                                               insert_view_name, table_name,
                                               fields, special_fields,
                                               set_up, clean_up,
                                               extra_statements):
            line(l)
        line("")


        line("CREATE OR REPLACE RULE %s" % insert_rule_name)
        line("AS ON INSERT TO %s" % insert_view_name)
        line("DO INSTEAD SELECT %s(NEW);" % insert_function_name)
        line("")
        self.add_teardown("DROP RULE %s ON %s;"
                          % (insert_rule_name, insert_view_name))

        line("GRANT INSERT ON %s TO %s;" % (insert_view_name, self.username))
        line("")

    def meta_to_field(self, meta):
        return Field(meta.getName(), self.sql_type(meta))

    def sql_type(self, meta):
        if isinstance(meta, data.NumericLeaf):
            return "NUMERIC"
        elif isinstance(meta, data.IntLeaf):
            return "INTEGER"
        elif isinstance(meta, data.DateLeaf):
            return "DATE"
        elif isinstance(meta, data.TextLeaf):
            length = meta.getMaxLength()
            if length:
                return "VARCHAR(%s)" % str(length)
            return "VARCHAR"
        elif isinstance(meta, data.TextAreaLeaf):
            return "TEXT"
        elif isinstance(meta, data.BoolLeaf):
            return "INTEGER"
        elif isinstance(meta, data.RadioNode):
            return "INTEGER"
        elif isinstance(meta, data.ChoiceNode):
            return "INTEGER"
        elif isinstance(meta, data.VirtualLeaf):
            return None
        elif isinstance(meta, ByteALeaf):
            return "BYTEA"
        elif isinstance(meta, DateTimeLeaf):
            return "TIMESTAMP"

        raise ValueError("Cannot determine SQL type for %r" % meta)


    def generate_insert_function(self, function_name, record_type, table,
                                 fields, special_fields, set_up, clean_up,
                                 extra_statements):

        field_names = list(chain((e[0] for e in special_fields),
                                 (field.name for field in fields)))
        field_exprs = list(chain((e[1] for e in special_fields),
                                 ("rec.%s" % field.name for field in fields)))

        yield "CREATE OR REPLACE FUNCTION"
        yield ("    %s(rec %s)" % (function_name, record_type))
        yield "RETURNS integer"
        yield "    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER"
        yield "AS $$"
        yield "DECLARE"
        yield "    rid integer;"
        yield "BEGIN"
        for stmt in set_up:
            yield "    " + stmt
        yield ("    INSERT INTO %s (" % table)
        yield (",\n".join("        %s" % field for field in field_names))
        yield "    )"
        yield "    VALUES ("
        yield (",\n".join("        %s" % expr for expr in field_exprs))
        yield "    )"
        yield ("    RETURNING id INTO rid;")
        yield ("    PERFORM set_uuid('%s', rid);" % table)
        for stmt in extra_statements:
            yield "    " + stmt
        for stmt in clean_up:
            yield "    " + stmt
        yield ("    RETURN rid;")
        yield "END;"
        yield "$$;"

        yield ("REVOKE ALL    ON FUNCTION %s(%s) FROM PUBLIC;"
               % (function_name, record_type))
        yield ("GRANT EXECUTE ON FUNCTION %s(%s) TO bewerbungsimport;"
               % (function_name, record_type))

        self.add_teardown("DROP FUNCTION %s(%s);"
                          % (function_name, record_type))
